﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace ClusteringBasic
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // Set maximum five clusters! Otherwise clusters won't be visualized
        // properly.
        public static int clnum = 3;

        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        public void runSample()
        {
            ClusterPlan plan = runClusterSample();
            if (plan != null)
                runMap(plan);
        }

        public ClusterPlan runClusterSample() 
        {
		    // Step 1: Instantiate remote interface for xClusters
            XClusterWSService client = new XClusterWSService();

		    // Step 2.1: Set CallerContext for adding log information into your basic
		    // response logs.
		    CallerContext cxt = new CallerContext();
		    cxt.log1 = "Basic Clustering";
		    // Step 2.2 Set CallerContext properties to define profile and CoordFormat
		    CallerContextProperty props1 = new CallerContextProperty 
            {
                key = "Profile",
                value = "default"
            };
		    CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
		    CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		    cxt.wrappedProperties = properties;

		    // Step 3: create object orders which contain a list of cluster orders
		    // (corresponds to one visit)
		    List<String> orders = new List<String>();
		    orders = loadOrder();
		    int max = orders.Count;
		    ClusterOrder[] clusterOrders = new ClusterOrder[max];

		    int index = 0;
		    foreach (String s in orders) {
                String[] splitString = s.Split(';');

                int oid = Convert.ToInt32(splitString[0]);
                int x = Convert.ToInt32(splitString[1]);
                int y = Convert.ToInt32(splitString[2]);

			    ClusterOrder order = new ClusterOrder();
			    order.orderID = oid;
			    
                Location location = new Location();
			    location.x = x;
			    location.y = y;
			    
                order.loc = location;
			    order.orderFixed = false;
			    order.quantity = 1;

			    clusterOrders[index] = order;
			    index++;
		    }

		    // Step 4: Define number of clusters and specific parameters
		    ClusterParams clusterParams = new ClusterParams
            {
                clusterCount = clnum,
                costPeriodMinute = 120,
                costPeriodMinuteSpecified = true,
                costDistanceKm = 360,
                costDistanceKmSpecified = true,
                constructionStep = true, // compactness is enabled
                constructionStepSpecified = true,
                improvementStep = false,
                improvementStepSpecified = true,
                postOptStep = false,
                postOptStepSpecified = true
            };
		    try {
			    // Step 5: Call method planClusters
			    ClusterPlan clusterPlan = client.planClusters(clusterOrders, clusterParams, cxt);
			
			    List<ClusterTableEntry> clusteringOrders = new List<ClusterTableEntry>();

			    foreach (OutputClusterOrder order in clusterPlan.wrappedOrders) {
				    ClusterTableEntry row = new ClusterTableEntry 
                    {
                        orderID = order.orderID,
                        cluster = order.clusterIndex
                    };
                    clusteringOrders.Add(row);
			    }

			    // visualize orders assigned to their clusters
			    gridCluster.ItemsSource = clusteringOrders;

                return clusterPlan;
		    } 
            catch (Exception e) 
            {
			    // TODO: exception handling
                return null;
		    } 
        }

        private List<String> loadOrder()
        {
            List<String> list = new List<String>();
            list.Add("101;671006;6382294");
            list.Add("102;681812;6375071");
            list.Add("103;682676;6388154");
            list.Add("104;675000;6380000");
            list.Add("105;679143;6374455");
            list.Add("106;684621;6374703");
            list.Add("107;684659;6372947");
            list.Add("108;683151;6371248");
            list.Add("109;685669;6371954");
            list.Add("110;683974;6369193");
            list.Add("111;683000;6366903");
            list.Add("112;678763;6367800");
            list.Add("113;681397;6369670");
            list.Add("114;678400;6370854");
            list.Add("115;674779;6373188");
            list.Add("116;678149;6380180");
            list.Add("117;680440;6381326");
            list.Add("118;665627;6378195");
            list.Add("119;687472;6386122");
            list.Add("120;666773;6381020");
            list.Add("121;687136;6382563");
            list.Add("122;665642;6382471");
            list.Add("123;665627;6386365");
            list.Add("124;672957;6388121");
            list.Add("125;677004;6388121");
            list.Add("126;682578;6384074");
            list.Add("127;687540;6379111");
            list.Add("128;681661;6384838");
            list.Add("129;687388;6385754");
            list.Add("130;685937;6388121");

            return list;
        }

        private void runMap(ClusterPlan clusterPlan) {
		    // Step 1: Initialize xmap client
		    XMapWSService xmapClient = new XMapWSService();

		    // Step 2: Set CallerContext
		    CallerContext cxt = new CallerContext();
		    cxt.log1 = "Basic Clustering";
		    // set CallerContext properties to define profile and CoordFormat
		    CallerContextProperty props1 = new CallerContextProperty
            {
                key = "profile",
                value = "default"
            };
		    CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
		    CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		    cxt.wrappedProperties = properties;

		    // Step 3: Create imageInfo and set relevant attributes
		    ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 530,
                width = 787,
                imageParameter = ""
            };

		    // Step 4: Realize object mapParams showing scale without use of miles.
		    MapParams mapParams = new MapParams
            {
                showScale = true,
                useMiles = false
            };

		    // Step 5: Create a bitmap array per cluster
		    int max = clusterPlan.wrappedOrders.Length;

		    Bitmap[] bitmapArray = new Bitmap[max];
		    int index = 0;
		    List<String> orders = loadOrder();

		    foreach (OutputClusterOrder clusterOrder in clusterPlan.wrappedOrders) {
			    String oid = "" + clusterOrder.orderID;

			    double x = 0.0;
			    double y = 0.0;
                xserver.Point point = new xserver.Point();
			    PlainPoint plainPoint = new PlainPoint();

			    foreach (String s in orders) {
                    String[] splitString = s.Split(';');
                    String orderIndex = splitString[0];

				    if (orderIndex == oid) {
                        x = Convert.ToInt32(splitString[1]);
                        y = Convert.ToInt32(splitString[2]);
					    plainPoint.x = x;
					    plainPoint.y = y;
					    point.point = plainPoint;
					    bitmapArray[index] = new Bitmap();
					    bitmapArray[index].position = point;

					    int cli = clusterOrder.clusterIndex;
					    if (cli == 1) {
						    bitmapArray[index].name = "flagblue.bmp";
						    bitmapArray[index].descr = "First cluster";
					    } else if (cli == 2) {
						    bitmapArray[index].name = "flagred.bmp";
						    bitmapArray[index].descr = "Second cluster";
					    } else if (cli == 3) {
						    bitmapArray[index].name = "flaggreen.bmp";
						    bitmapArray[index].descr = "Third cluster";
					    } else if (cli == 4) {
						    bitmapArray[index].name = "n_marker1.bmp";
						    bitmapArray[index].descr = "Forth cluster";
					    } else if (cli == 5) {
						    bitmapArray[index].name = "n_marker2.bmp";
						    bitmapArray[index].descr = "Fifth cluster";
					    } else {
						    bitmapArray[index].name = "location.bmp";
						    bitmapArray[index].descr = "Not defined";
					    }

					    index++;
					    break;
				    }
			    }
		    }

		    // Step 6: Set bitmapOptions
		    BitmapOptions bitmapOptions = new BitmapOptions();
		    // change the reference point for the pixel position of the bitmap
		    // by default the bitmap center will be used.
		    // here: bitmap displacement of 20 pixels.
            PlainPoint referencePoint = new PlainPoint { x = 20, y = 20 };
		    bitmapOptions.referencePoint = referencePoint;
		    bitmapOptions.transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 };

		    // Step 7: Add bitmapArray and bitmapOptions into bitmaps
		    Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps { wrappedBitmaps = bitmapArray, options = bitmapOptions } };

		    // Step 8: Initialize customLayer with the relevant settings
		    CustomLayer customLayer = new CustomLayer
            {
                centerObjects = true,
                drawPriority = 25000,
                name = "ObjectLayer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps
            };

		    // Step 9: Put customLayer into layers
		    Layer[] layers = new Layer[] { customLayer };

		    // Step 10: Define boolean value including image in response
		    bool includeImageInResponse = false;

		    try 
            {
			    // Step 11: Return map object with its characteristics
			    Map map = xmapClient.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse, cxt);

			    // Step 12: Retrieve icon
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage image = new BitmapImage();

                // Display BitmapImage
                image.BeginInit();
                image.UriSource = new Uri(imageUrl);
                image.EndInit();
                img1.Source = image;
		    } 
            catch (Exception e) 
            {
			    // proper exception handling: see API for PTV's exception handling!
		    }
	    }
    }

    public class ClusterTableEntry
    {
        public int orderID { get; set; }
        public int cluster { get; set; }
    }
}
