﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace ClusteringPlanMultiWeek
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        public void runSample()
        {
            MultiWeeksPlan plan = runClusterSample();
            if (plan != null)
                runMap(plan);
        }

        public MultiWeeksPlan runClusterSample()
        {
            // Step 1: Instantiate remote interface for xCluster
            XClusterWSService client = new XClusterWSService();

            // Step 2.1: Set CallerContext for adding log information into your basic
            // response logs.
            CallerContext cxt = new CallerContext();
            cxt.log1 = "Plan Week";
            // Step 2.2 Set CallerContext properties to define profile and CoordFormat
            CallerContextProperty props1 = new CallerContextProperty
            {
                key = "Profile",
                value = "default"
            };
            CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
            CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
            cxt.wrappedProperties = properties;

            // Step 3: create object orders which contain a list of cluster orders
            // (corresponds to one visit)
            List<String> orders = new List<String>();
            orders = loadOrder();
            int max = orders.Count;
            MultiWeeksOrder[] multiWeeksOrders = new MultiWeeksOrder[max];

            int index = 0;
            foreach (String s in orders)
            {
                String[] splitString = s.Split(';');

                int oid = Convert.ToInt32(splitString[0]);
                int x = Convert.ToInt32(splitString[1]);
                int y = Convert.ToInt32(splitString[2]);
                int wmax = Convert.ToInt32(splitString[3]);
                String[] vecVisitsStrings = splitString[4].Split('~');
                List<int> vecVisits = new List<int>();
                foreach (String s1 in vecVisitsStrings)
                {
                    vecVisits.Add(Convert.ToInt32(s1));
                }


                // Define location
                Location location = new Location();
                location.x = x;
                location.y = y;

                // Define validity
                WeekValidity validity1 = new WeekValidity();
                validity1.day1 = false;
                validity1.day2 = true;
                validity1.day3 = true;
                validity1.day4 = true;
                validity1.day5 = true;
                validity1.day6 = true;
                validity1.day7 = false;

                WeekValidity validity2 = new WeekValidity();
                validity2.day1 = true;
                validity2.day2 = true;
                validity2.day3 = true;
                validity2.day4 = true;
                validity2.day5 = true;
                validity2.day6 = false;
                validity2.day7 = false;

                WeekValidity[] validities = new WeekValidity[] { validity1, validity2 };

                // Define all possible weekday patterns according to the visit rhythm
                WeekValidity[] weekdayPatterns = setWeekdayPatterns(wmax);

                // Set assignmentRules
                MultiWeeksRules assignmentRules = new MultiWeeksRules();
                assignmentRules.wrappedValidities = validities;
                assignmentRules.weekRhythm = 2;
                assignmentRules.firstWeekFix = false;
                assignmentRules.visitsPerWeek = wmax;
                assignmentRules.wrappedWeekdayPatterns = weekdayPatterns;

                // Set quantity to weekVisits
                MultiWeeksVisit[] multiWeeksVisits = new MultiWeeksVisit[wmax];
                int oindex = 0;
                foreach (int visit in vecVisits)
                {
                    int quantity = visit;
                    MultiWeeksVisit multiWeeksVisit = new MultiWeeksVisit();
                    multiWeeksVisit.quantity = quantity;
                    multiWeeksVisits[oindex] = multiWeeksVisit;
                    oindex++;
                }

                MultiWeeksOrder order = new MultiWeeksOrder
                {
                    orderID = oid,
                    loc = location,
                    orderFixed = false,
                    wrappedVisits = multiWeeksVisits,
                    assignmentRules = assignmentRules
                };

                multiWeeksOrders[index] = order;
                index++;
            }

            // Step 4: Define number of clusters and specific parameters
            MultiWeeksParams multiWeeksParams = new MultiWeeksParams
            {
                weekCount = 2,
                weekdayCount = 6,
                weekdayCountSpecified = true,
                costPeriodMinute = 400,
                costPeriodMinuteSpecified = true,
                costDistanceKm = 100,
                costDistanceKmSpecified = true,
                constructionStep = true,
                constructionStepSpecified = true,
                improvementStep = true,
                improvementStepSpecified = true,
                postOptStep = true,
                postOptStepSpecified = true,
            };

            WeekWeight weight1 = new WeekWeight
            {
                day1 = 20,
                day2 = 20,
                day3 = 20,
                day4 = 20,
                day5 = 20,
                day6 = 20,
                day7 = 0,
            };

            WeekWeight weight2 = new WeekWeight
            {
                day1 = 20,
                day2 = 20,
                day3 = 20,
                day4 = 20,
                day5 = 20,
                day6 = 20,
                day7 = 0,
            };

            WeekWeight[] weekWeights = new WeekWeight[] { weight1, weight2 };
            multiWeeksParams.wrappedWeights = weekWeights;

            try
            {
                // Step 5: Call method planWeek
                MultiWeeksPlan multiWeeksPlan = client.planMultiWeeks(multiWeeksOrders, multiWeeksParams, cxt);

                createTableOutputMultiWeekOrder(multiWeeksPlan);
                createTableWeekdayCluster(multiWeeksPlan);
                return multiWeeksPlan;
            }
            catch (Exception e)
            {
                // TODO: exception handling
                return null;
            }
        }

        private void createTableOutputMultiWeekOrder(MultiWeeksPlan weekPlan)
        {
            List<OrderTableEntry> weekOrders = new List<OrderTableEntry>();

            foreach (OutputMultiWeeksOrder order in weekPlan.wrappedOrders)
            {
                String weekday = "";
                String week = "";
                foreach (OutputMultiWeeksVisit visit in order.wrappedVisits)
                {
                    int w = visit.week;
                    if (w == 1)
                    {
                        week = week + "WW 1";
                    }
                    else
                    {
                        week = week + "WW 2";
                    }
                    week = week + "; ";

                    int wd = visit.weekday;
                    if (wd == 1)
                    {
                        weekday = weekday + "Monday";
                    }
                    else if (wd == 2)
                    {
                        weekday = weekday + "Tuesday";
                    }
                    else if (wd == 3)
                    {
                        weekday = weekday + "Wednesday";
                    }
                    else if (wd == 4)
                    {
                        weekday = weekday + "Thursday";
                    }
                    else if (wd == 4)
                    {
                        weekday = weekday + "Friday";
                    }
                    else
                    {
                        weekday = weekday + "Saturday";
                    }
                    weekday = weekday + "; ";
                }

                OrderTableEntry row = new OrderTableEntry
                {
                    orderID = order.orderID,
                    week = week,
                    weekday = weekday
                };

                weekOrders.Add(row);
            }
            gridOrders.ItemsSource = weekOrders;
        }

        private void createTableWeekdayCluster(MultiWeeksPlan weekPlan)
        {
            List<WeekdayTableEntry> weekClusters = new List<WeekdayTableEntry>();

            foreach (DayCluster cluster in weekPlan.wrappedClusters)
            {
                int wd = cluster.weekday;
                String weekday = "";
                if (wd == 1)
                {
                    weekday = "Monday";
                }
                else if (wd == 2)
                {
                    weekday = "Tuesday";
                }
                else if (wd == 3)
                {
                    weekday = "Wednesday";
                }
                else if (wd == 4)
                {
                    weekday = "Thursday";
                }
                else if (wd == 5)
                {
                    weekday = "Friday";
                }
                else
                {
                    weekday = "Saturday";
                }

                int w = cluster.week;
                String week = "";
                if (w == 1)
                {
                    week = "Week Number 1";
                }
                else
                {
                    week = "Week Number 2";
                }

                WeekdayTableEntry row = new WeekdayTableEntry
                {
                    load = cluster.load,
                    servicePeriod = cluster.quantity,
                    target = cluster.target,
                    visitCount = cluster.visitCount,
                    weekday = weekday,
                    week = week
                };
                weekClusters.Add(row);
            }

            // visualize orders assigned to their clusters
            gridWeekdays.ItemsSource = weekClusters;
        }

        private static WeekValidity[] setWeekdayPatterns(int wmax)
        {
            WeekValidity[] weekdayPatterns = null;

            WeekValidity weekdayPattern1 = new WeekValidity();
            weekdayPattern1.day1 = true;
            weekdayPattern1.day2 = false;
            weekdayPattern1.day3 = false;
            weekdayPattern1.day4 = false;
            weekdayPattern1.day5 = false;
            weekdayPattern1.day6 = false;
            weekdayPattern1.day7 = false;

            WeekValidity weekdayPattern2 = new WeekValidity();
            weekdayPattern2.day1 = false;
            weekdayPattern2.day2 = true;
            weekdayPattern2.day3 = false;
            weekdayPattern2.day4 = false;
            weekdayPattern2.day5 = false;
            weekdayPattern2.day6 = false;
            weekdayPattern2.day7 = false;

            WeekValidity weekdayPattern3 = new WeekValidity();
            weekdayPattern3.day1 = false;
            weekdayPattern3.day2 = false;
            weekdayPattern3.day3 = true;
            weekdayPattern3.day4 = false;
            weekdayPattern3.day5 = false;
            weekdayPattern3.day6 = false;
            weekdayPattern3.day7 = false;

            WeekValidity weekdayPattern4 = new WeekValidity();
            weekdayPattern4.day1 = false;
            weekdayPattern4.day2 = false;
            weekdayPattern4.day3 = false;
            weekdayPattern4.day4 = true;
            weekdayPattern4.day5 = false;
            weekdayPattern4.day6 = false;
            weekdayPattern4.day7 = false;

            WeekValidity weekdayPattern5 = new WeekValidity();
            weekdayPattern5.day1 = false;
            weekdayPattern5.day2 = false;
            weekdayPattern5.day3 = false;
            weekdayPattern5.day4 = false;
            weekdayPattern5.day5 = true;
            weekdayPattern5.day6 = false;
            weekdayPattern5.day7 = false;

            WeekValidity weekdayPattern6 = new WeekValidity();
            weekdayPattern6.day1 = false;
            weekdayPattern6.day2 = false;
            weekdayPattern6.day3 = false;
            weekdayPattern6.day4 = false;
            weekdayPattern6.day5 = false;
            weekdayPattern6.day6 = true;
            weekdayPattern6.day7 = false;

            WeekValidity weekdayPattern7 = new WeekValidity();
            weekdayPattern7.day1 = true;
            weekdayPattern7.day2 = false;
            weekdayPattern7.day3 = true;
            weekdayPattern7.day4 = false;
            weekdayPattern7.day5 = false;
            weekdayPattern7.day6 = false;
            weekdayPattern7.day7 = false;

            WeekValidity weekdayPattern8 = new WeekValidity();
            weekdayPattern8.day1 = false;
            weekdayPattern8.day2 = true;
            weekdayPattern8.day3 = false;
            weekdayPattern8.day4 = true;
            weekdayPattern8.day5 = false;
            weekdayPattern8.day6 = false;
            weekdayPattern8.day7 = false;

            WeekValidity weekdayPattern9 = new WeekValidity();
            weekdayPattern9.day1 = false;
            weekdayPattern9.day2 = false;
            weekdayPattern9.day3 = true;
            weekdayPattern9.day4 = false;
            weekdayPattern9.day5 = true;
            weekdayPattern9.day6 = false;
            weekdayPattern9.day7 = false;

            WeekValidity weekdayPattern10 = new WeekValidity();
            weekdayPattern10.day1 = false;
            weekdayPattern10.day2 = false;
            weekdayPattern10.day3 = false;
            weekdayPattern10.day4 = true;
            weekdayPattern10.day5 = false;
            weekdayPattern10.day6 = true;
            weekdayPattern10.day7 = false;

            WeekValidity weekdayPattern11 = new WeekValidity();
            weekdayPattern11.day1 = true;
            weekdayPattern11.day2 = false;
            weekdayPattern11.day3 = false;
            weekdayPattern11.day4 = true;
            weekdayPattern11.day5 = false;
            weekdayPattern11.day6 = false;
            weekdayPattern11.day7 = false;

            WeekValidity weekdayPattern12 = new WeekValidity();
            weekdayPattern12.day1 = false;
            weekdayPattern12.day2 = true;
            weekdayPattern12.day3 = false;
            weekdayPattern12.day4 = false;
            weekdayPattern12.day5 = true;
            weekdayPattern12.day6 = false;
            weekdayPattern12.day7 = false;

            WeekValidity weekdayPattern13 = new WeekValidity();
            weekdayPattern13.day1 = false;
            weekdayPattern13.day2 = false;
            weekdayPattern13.day3 = true;
            weekdayPattern13.day4 = false;
            weekdayPattern13.day5 = false;
            weekdayPattern13.day6 = true;
            weekdayPattern13.day7 = false;

            WeekValidity weekdayPattern14 = new WeekValidity();
            weekdayPattern14.day1 = true;
            weekdayPattern14.day2 = false;
            weekdayPattern14.day3 = false;
            weekdayPattern14.day4 = false;
            weekdayPattern14.day5 = true;
            weekdayPattern14.day6 = false;
            weekdayPattern14.day7 = false;

            WeekValidity weekdayPattern15 = new WeekValidity();
            weekdayPattern15.day1 = false;
            weekdayPattern15.day2 = true;
            weekdayPattern15.day3 = false;
            weekdayPattern15.day4 = false;
            weekdayPattern15.day5 = false;
            weekdayPattern15.day6 = true;
            weekdayPattern15.day7 = false;

            if (wmax == 1)
            {
                weekdayPatterns = new WeekValidity[] { weekdayPattern1,
					weekdayPattern2, weekdayPattern3, weekdayPattern4,
					weekdayPattern5, weekdayPattern6 };
            }
            else
            {
                weekdayPatterns = new WeekValidity[] { weekdayPattern7,
					weekdayPattern8, weekdayPattern9, weekdayPattern10,
					weekdayPattern11, weekdayPattern12, weekdayPattern13,
					weekdayPattern14, weekdayPattern15 };
            }

            return weekdayPatterns;
        }

        private static List<String> loadOrder()
        {
            List<String> list = new List<String>();
            // orderId, X, Y, number of week visits, visits values (service period(s)
            // and day(s))
            list.Add("101;671006;6382294;1;90");
            list.Add("102;681812;6375071;1;120");
            list.Add("103;682676;6388154;1;90");
            list.Add("104;675000;6380000;1;70");
            list.Add("105;679143;6374455;2;90~110");
            list.Add("106;684621;6374703;1;180");
            list.Add("107;684659;6372947;1;90");
            list.Add("108;683151;6371248;1;50");
            list.Add("109;685669;6371954;1;60");
            list.Add("110;683974;6369193;2;60~80");
            list.Add("111;683000;6366903;1;90");
            list.Add("112;678763;6367800;1;120");
            list.Add("113;681397;6369670;1;110");
            list.Add("114;678400;6370854;1;90");
            list.Add("115;674779;6373188;2;60~45");
            list.Add("116;678149;6380180;1;45");
            list.Add("117;680440;6381326;1;30");
            list.Add("118;665627;6378195;1;90");
            list.Add("119;687472;6386122;1;180");
            list.Add("120;666773;6381020;2;120~110");
            list.Add("121;687136;6382563;1;60");
            list.Add("122;665642;6382471;1;30");
            list.Add("123;665627;6386365;1;45");
            list.Add("124;672957;6388121;1;60");
            list.Add("125;677004;6388121;2;80~100");
            list.Add("126;682578;6384074;1;90");
            list.Add("127;687540;6379111;1;100");
            list.Add("128;681661;6384838;1;120");
            list.Add("129;687388;6385754;1;130");
            list.Add("130;685937;6388121;2;150~130");
            list.Add("131;683000;6366903;1;90");
            list.Add("132;679763;6369800;1;120");
            list.Add("133;683397;6366670;1;110");
            list.Add("134;672400;6378854;1;90");
            list.Add("135;679779;6370188;2;60~45");
            list.Add("136;688149;6382180;1;45");
            list.Add("137;681440;6380326;1;30");
            list.Add("138;668627;6368195;1;90");
            list.Add("139;677472;6384122;1;180");
            list.Add("140;676773;6382020;2;120~110");
            list.Add("141;689136;6384563;1;60");
            list.Add("142;665042;6392471;1;30");
            list.Add("143;662627;6382365;1;45");
            list.Add("144;675957;6380121;1;60");
            list.Add("145;676004;6384121;2;80~100");
            list.Add("146;680578;6389074;1;90");
            list.Add("147;682540;6359111;1;100");
            list.Add("148;688661;6389838;1;120");
            list.Add("149;680388;6380754;1;130");
            list.Add("150;680937;6380121;2;150~130");

            return list;
        }

        private CallerContext setCallerContext()
        {
            CallerContext cxt = new CallerContext();
            cxt.log1 = "Plan Week";
            // set CallerContext properties to define profile and CoordFormat
            CallerContextProperty props1 = new CallerContextProperty
            {
                key = "Profile",
                value = "default"
            };
            CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
            CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
            cxt.wrappedProperties = properties;
            return cxt;
        }

        private static ImageInfo setImageInfo()
        {
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 520,
                width = 787,
                imageParameter = ""
            };
            return imageInfo;
        }

        private static MapParams setMapParams()
        {
            MapParams mapParams = new MapParams
            {
                showScale = true,
                useMiles = false
            };
            return mapParams;
        }

        private static CustomLayer setCustomLayer()
        {
            CustomLayer customLayer = new CustomLayer
            {
                centerObjects = true,
                drawPriority = 25000,
                name = "ObjectLayer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true
            };
            return customLayer;
        }

        private void runMap(MultiWeeksPlan multiWeeksPlan)
        {
            // Step 1: Initialize xmap client
            XMapWSService xmapClient = new XMapWSService();

            // Step 2: Set CallerContext
            CallerContext cxt = setCallerContext();

            // Step 3: Create imageInfo and set relevant attributes
            ImageInfo imageInfo = setImageInfo();

            // Step 4: Realize object mapParams showing scale without use of miles.
            MapParams mapParams = setMapParams();

            // Step 5: Define boolean value including image in response
            bool includeImageInResponse = false;

            // Step 6: Initialize customLayer with the relevant settings
            CustomLayer customLayer = setCustomLayer();

            // Step 7.1: Create bitmap arrays per weekday
            int num = multiWeeksPlan.wrappedClusters.Length;

		    for (int i = 0; i < num; i++) {
			    int max = multiWeeksPlan.wrappedClusters[i].visitCount;
			    if (max == 0) {
				    continue;
			    }
			    Bitmap[] bitmapArray = new Bitmap[max];
			    List<String> orders = loadOrder();
			    int index = 0;
			    int weeknumber;
			    int countClusterDay;
			    String weekDay = "";
			    if ((i == 0)) {
				    weekDay = "Monday-1";
				    weeknumber = 1;
				    countClusterDay = 1;
			    } else if ((i == 1)) {
				    weekDay = "Tuesday-1";
				    weeknumber = 1;
				    countClusterDay = 2;
			    } else if ((i == 2)) {
				    weekDay = "Wednesday-1";
				    weeknumber = 1;
				    countClusterDay = 3;
			    } else if ((i == 3)) {
				    weekDay = "Thursday-1";
				    weeknumber = 1;
				    countClusterDay = 4;
			    } else if ((i == 4)) {
				    weekDay = "Friday-1";
				    weeknumber = 1;
				    countClusterDay = 5;
			    } else if ((i == 5)) {
				    weekDay = "Saturday-1";
				    weeknumber = 1;
				    countClusterDay = 6;
			    } else if ((i == 6)) {
				    weekDay = "Monday-2";
				    weeknumber = 2;
				    countClusterDay = 1;
			    } else if ((i == 7)) {
				    weekDay = "Tuesday-2";
				    weeknumber = 2;
				    countClusterDay = 2;
			    } else if ((i == 8)) {
				    weekDay = "Wednesday-2";
				    weeknumber = 2;
				    countClusterDay = 3;
			    } else if ((i == 9)) {
				    weekDay = "Thursday-2";
				    weeknumber = 2;
				    countClusterDay = 4;
			    } else if ((i == 10)) {
				    weekDay = "Friday-2";
				    weeknumber = 2;
				    countClusterDay = 5;
			    } else {
				    weekDay = "Saturday-2";
				    countClusterDay = 6;
				    weeknumber = 2;
			    }

			    foreach (OutputMultiWeeksOrder multiWeeksOrder in multiWeeksPlan.wrappedOrders) {

				    // Step 7.2: Plot multi-week visits
				    foreach (OutputMultiWeeksVisit outputMultiWeeksVisit in multiWeeksOrder.wrappedVisits) {
					    int ww = outputMultiWeeksVisit.week;

					    // Step 7.3: Get location from order and fill
					    // bitmapArray
					    if ((outputMultiWeeksVisit.weekday == countClusterDay) && (ww == weeknumber)) {
						    String oid = "" + multiWeeksOrder.orderID;

						    double x = 0.0;
						    double y = 0.0;
                            xserver.Point point = new xserver.Point();
						    PlainPoint plainPoint = new PlainPoint();

						    foreach (String s in orders) {
                                String[] splitString = s.Split(';');
                                String orderIndex = splitString[0];
							    if (orderIndex == oid) {
								    x = Convert.ToInt32(splitString[1]);
                                    y = Convert.ToInt32(splitString[2]);
								    plainPoint.x = x;
								    plainPoint.y = y;
								    point.point = plainPoint;
								    bitmapArray[index] = new Bitmap();
								    bitmapArray[index].position = point;

								    if ((countClusterDay == 1)
										    || (countClusterDay == 7)) {
									    bitmapArray[index].name = "flagblue.bmp";
									    bitmapArray[index].descr = oid;
								    } else if ((countClusterDay == 2)
										    || (countClusterDay == 8)) {
									    bitmapArray[index].name = "flagred.bmp";
									    bitmapArray[index].descr = oid;
								    } else if ((countClusterDay == 3)
										    || (countClusterDay == 9)) {
									    bitmapArray[index].name = "flaggreen.bmp";
									    bitmapArray[index].descr = oid;
								    } else if ((countClusterDay == 4)
										    || (countClusterDay == 10)) {
									    bitmapArray[index].name = "n_marker1.bmp";
									    bitmapArray[index].descr = oid;
								    } else if ((countClusterDay == 5)
										    || (countClusterDay == 11)) {
									    bitmapArray[index].name = "n_marker2.bmp";
									    bitmapArray[index].descr = oid;
								    } else {
									    bitmapArray[index].name = "location.bmp";
									    bitmapArray[index].descr = oid;
								    }

								    index++;
								    break;
							    }
						    }
					    }
				    }
			    }
                // Step 8.1: Set bitmapOptions
                BitmapOptions bitmapOptions = new BitmapOptions();
                PlainPoint referencePoint = new PlainPoint { x = 20, y = 20 };
                bitmapOptions.referencePoint = referencePoint;
                bitmapOptions.transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 };

                // Step 8.2: Add bitmapArray and bitmapOptions into bitmaps
                Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps { wrappedBitmaps = bitmapArray, options = bitmapOptions } };
                customLayer.wrappedBitmaps = bitmaps;

                // Step 9: Put customLayer into layers
                Layer[] layers = new Layer[] { customLayer };

                try
                {
                    // Step 10: Return map object with its characteristics
                    Map map = xmapClient.renderMap(null, mapParams, imageInfo, layers, includeImageInResponse, cxt);

                    // Step 11: Retrieve icon
                    String imageUrl = "http://" + map.image.url;
                    // Create BitmapImage
                    BitmapImage image = new BitmapImage();

                    // Display BitmapImage
                    image.BeginInit();
                    image.UriSource = new Uri(imageUrl);
                    image.EndInit();
                    switch (i)
                    {
                        case 0: { img1.Source = image; break; }
                        case 1: { img2.Source = image; break; }
                        case 2: { img3.Source = image; break; }
                        case 3: { img4.Source = image; break; }
                        case 4: { img5.Source = image; break; }
                        case 5: { img6.Source = image; break; }
                        case 6: { img7.Source = image; break; }
                        case 7: { img8.Source = image; break; }
                        case 8: { img9.Source = image; break; }
                        case 9: { img10.Source = image; break; }
                        case 10: { img11.Source = image; break; }
                        case 11: { img12.Source = image; break; }
                    }

                }
                catch (Exception e)
                {
                    // proper exception handling: see API for PTV's exception handling!
                }

            }
        }
    }

    public class OrderTableEntry
    {
        public int orderID { get; set; }
        public String week { get; set; }
        public String weekday { get; set; }
    }

    public class WeekdayTableEntry
    {
        public double load { get; set; }
        public double servicePeriod { get; set; }
        public double target { get; set; }
        public int visitCount { get; set; }
        public String weekday { get; set; }
        public String week { get; set; }
    }
}
