﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GeocodingWithAdditionalFields.XLocate;

namespace GeocodingBasic
{
    class Program
    {
        static void Main(string[] args)
        {
            // Step 1: Instantiate XLocate client
            XLocateWSService client = new XLocateWSService();

            // Step 2.1: Set CallerContext for adding log information into your basic response logs.
            CallerContext context = new CallerContext();
            context.log1 = "Geocoding with additional fields";

            // Step 2.2 Set CallerContext properties to define profile and CoordFormat
            CallerContextProperty prop1 = new CallerContextProperty();
            prop1.key = "Profile";
            prop1.value = "default";
            CallerContextProperty prop2 = new CallerContextProperty();
            prop2.key = "CoordFormat";
            prop2.value = "PTV_MERCATOR";
            CallerContextProperty[] props = new CallerContextProperty[] {prop1, prop2};
            context.wrappedProperties = props;

            // Step 3: Create address object
            Address addr = new Address();
            addr.country = "L";
            addr.postCode = "1856";
            addr.city = "Luxemburg";
            addr.street = "Rue Evrard Ketten";
            addr.houseNumber = "40";
			
			// Step 4: Create an array of additional result fields
			AdditionalField addfield = new AdditionalField();
            addfield.field = ResultField.DETAILLEVEL;
			ResultField resfield1 = addfield.field;
			addfield = new AdditionalField();
            addfield.field = ResultField.DETAILLEVEL_DESCRIPTION;
			ResultField resfield2 = addfield.field;
			addfield = new AdditionalField();
            addfield.field = ResultField.SCORE_TOTALSCORE;
			ResultField resfield3 = addfield.field;
			addfield = new AdditionalField();
            addfield.field = ResultField.CLASSIFICATION;
			ResultField resfield4 = addfield.field;
			addfield = new AdditionalField();
            addfield.field = ResultField.CLASSIFICATION_DESCRIPTION;
			ResultField resfield5 = addfield.field;

			ResultField[] resfield = {resfield1,
					resfield2, 	resfield3,
					resfield4,	resfield5};
			
            try
            {
                // Step 5: Get address response, pass the CallerContext and the result field as a parameter
                AddressResponse response = client.findAddress(addr, null, null, resfield, context);

                // Step 6: Output the results to the console
                foreach (ResultAddress ra in response.wrappedResultList)
                {
                    Console.WriteLine(
                        String.Format("Found {0}~{1}~{2}~{3}~{4}, Detail Level: {5}, Detail Level Description: {6}, Total Score: {7}, Classification: {8}, Classification Description: {9}",
                            ra.country, ra.postCode, ra.city, ra.street, ra.houseNumber,
                            ra.wrappedAdditionalFields[0].value,
                            ra.wrappedAdditionalFields[1].value,
                            ra.wrappedAdditionalFields[2].value,
                            ra.wrappedAdditionalFields[3].value,
                            ra.wrappedAdditionalFields[4].value)
                    );
                }
                Console.WriteLine("Press any key to exit...");
                Console.ReadLine();
            }
            catch (Exception e)
            {
                //TODO: proper exception handling
            }
        }
    }
}
