﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ReverseGeocodingBasic.XLocate;

namespace ReverseGeocodingBasic
{
    class Program
    {
        static void Main(string[] args)
        {
            // Step 1: Instantiate XLocate client
            XLocateWSService client = new XLocateWSService();

            // Step 2: Set location
            PlainPoint plainPoint = new PlainPoint();
            plainPoint.x = 611767.0;
            plainPoint.y = 4962153.0;

            Point point = new Point();
            point.point = plainPoint;

            Location location = new Location();
            location.coordinate = point;

            // Step 3.1: Set CallerContext
            CallerContext context = new CallerContext();
            context.log1 = "Basic Reverse Geocoding";

            // Step 3.2: Set CallerContext properties to define profile and CoordFormat
            //           Use PTV_GEODECIMAL according to location's coordinate format
            CallerContextProperty prop1 = new CallerContextProperty();
            prop1.key = "Profile";
            prop1.value = "default";
            CallerContextProperty prop2 = new CallerContextProperty();
            prop2.key = "CoordFormat";
            prop2.value = "PTV_GEODECIMAL";
            CallerContextProperty[] props = new CallerContextProperty[] { prop1, prop2 };
            context.wrappedProperties = props;

            // Step 4.1: Create object ReverseSearchOption 
            //           Set maximum size <= 5 for address list and tolerance: default value = 160
            ReverseSearchOption soEngineTargetSize = new ReverseSearchOption();
            soEngineTargetSize.value = "5";
            soEngineTargetSize.param = ReverseSearchParameter.ENGINE_TARGETSIZE;
            ReverseSearchOption soEngineTolerance = new ReverseSearchOption();
            soEngineTolerance.value = "160";
            soEngineTolerance.param = ReverseSearchParameter.ENGINE_TOLERANCE;

            // Step 4.2: Create a new array SearchOptions that contains defined reverse search options.
            ReverseSearchOption[] options = new ReverseSearchOption[] { soEngineTargetSize, soEngineTolerance };

            try
            {
                // Step 5: Get address response, pass the SearchOptions as a parameter
                AddressResponse response =
                    client.findLocation(location, options, null, null, context);

                // Step 6: Output the results to the console
                foreach (ResultAddress ra in response.wrappedResultList)
                {
                    Console.WriteLine(
                        String.Format("Found {0}~{1}~{2}~{3}~{4} at location: x={5} y={6}",
                            ra.country, ra.postCode, ra.city, ra.street, ra.houseNumber, ra.coordinates.point.x, ra.coordinates.point.y)
                    );
                }
                Console.WriteLine("Press any key to exit...");
                Console.ReadLine();
            }
            catch (Exception e)
            {
                //TODO: proper exception handling
            }
        }
    }
}
