﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingWithCustomLayer.XMap;

namespace MappingWithCustomLayer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }
        public void runSample()
        {
            // step 1:
            // instantiate remote interface for xMap
            XMapWSService client = new XMapWSService();

            // Step 2.1: define upper left coordinate of the bounding box
            MappingWithCustomLayer.XMap.Point leftTop = new MappingWithCustomLayer.XMap.Point();
            PlainPoint plainPointLeftTop = new PlainPoint { x = 677712.4829360023, y = 6378309.416843999 };
            leftTop.point = plainPointLeftTop;

            // Step 2.2: define lower right coordinate of the bounding box
            MappingWithCustomLayer.XMap.Point rightBottom = new MappingWithCustomLayer.XMap.Point();
            PlainPoint plainPointRightBottom = new PlainPoint {x = 693103.0205360018, y = 6366766.513643999 };
            rightBottom.point = plainPointRightBottom;

            // Step 2.3: instantiate boundingBox and use the proper corners.
            BoundingBox boundingBox = new BoundingBox();
            boundingBox.leftTop = leftTop;
            boundingBox.rightBottom = rightBottom;

            // Step 3: set properties for class imageInfo 
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 480;
            imageInfo.width = 640;
            imageInfo.imageParameter = "";

            // Step 4: set scale and avoid miles
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;

            // Step 5: don't include image in response
            bool includeImageInResponse = false;

            // Step 6.1: Prepare plainPoints
            PlainPoint[] plainPoints = new PlainPoint[5];
            plainPoints[0] = new PlainPoint { x = 681854.0, y = 6372751.0 };
            plainPoints[1] = new PlainPoint { x = 680941.0, y = 6370524.0 };
            plainPoints[2] = new PlainPoint { x = 683598.0, y = 6371416.0 };
            plainPoints[3] = new PlainPoint { x = 683165.0, y = 6374963.0 };
            plainPoints[4] = new PlainPoint { x = 679546.0, y = 6373244.0 };

            // Step 6.2: Create plainLineString
            PlainLineString plainLineString = new PlainLineString { wrappedPoints = plainPoints };

            // Step 6.3: Set lineStrings
            LineString[] lineStrings = new LineString[1];
            lineStrings[0] = new LineString { lineString = plainLineString };

            // Step 6.4 Add main line combined with part options	
            LineOptions lineOptions = new LineOptions();
            LinePartOptions partoptions = new LinePartOptions();
            partoptions.visible = true;
            partoptions.color = new MappingWithCustomLayer.XMap.Color { red = 255, green = 50, blue = 20 };
            partoptions.width = 7;
            lineOptions.mainLine = partoptions;

            // Step 6.5: Define lines and put lineStrings and options togehter
            Lines[] lines = new Lines[1];
            lines[0] = new Lines { wrappedLines = lineStrings, options = lineOptions };

            // Step 7.1: Define customLayer that contains lines and further attributes
            CustomLayer customLayer = new CustomLayer();
            customLayer.visible = true;
            customLayer.drawPriority = 25000;
            customLayer.wrappedLines = lines;
            customLayer.objectInfos = ObjectInfoType.NONE;

            // Step 7.2: Set customLayer
            Layer[] layers = new Layer[] { customLayer };

            try
            {
                // Step 8: call renderMapBoundingBox and return a valid object map.
                Map map = client.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 9: retrieve and display the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
            }
            catch (Exception e)
            {
                // TODO proper exception handling
            }
        }
    }
}
