﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingWithGeometryLayer.XMap;

namespace MappingWithGeometryLayer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            
            // Step 1: Initialize xmap client
            XMapWSService xMapClient = new XMapWSService();
            // Note: the Germany map is required as minimum in this example!
            // Step 2: Set center
            XMap.Point point = new XMap.Point { point = new PlainPoint { x = 850000, y = 6900000 } };

            // Step 3: Set the attributes for mapSection
            MapSection mapSection = new MapSection();
            mapSection.center = point;
            mapSection.scale = 20000;

            // Step 4: Set the attributes for mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;

            // Step 5: Set the attributes for imageInfo
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 6: Set value to false
            bool includeImageInResponse = false;

            // Step 7: Define both polygons in WKT-format
            XMap.Polygon polygon1 = new XMap.Polygon();
            polygon1.wkt = "POLYGON((800000 6860000, 800000 6880000, 900000 6880000, 900000 6860000, 800000 6860000)" +
                        ",(820000 6862000, 820000 6868000, 880000 6868000, 880000 6862000, 820000 6862000))";

            /**
            XMap.Polygon polygon2 = new XMap.Polygon();
            polygon2.wkt = "POLYGON((800000 6900000, 900000 6900000, 850000 7000000, 800000 6900000)" +
                        ",(830000 6910000, 870000 6910000, 850000 6950000, 830000 6910000))";
            **/

            // An alternative to using WKT is to use the available data structures
            // For polygon2 this would look like:
            XMap.Polygon polygon2 = new XMap.Polygon
            {
                polygon =
                    new PlainPolygon
                    {
                        wrappedLinearRings =
                            new PlainLinearRing[]{
                                new PlainLinearRing { wrappedPoints =
                                        new PlainPoint[]{
                                                new PlainPoint { x = 800000, y = 6900000 },
                                                new PlainPoint { x = 900000, y = 6900000 },
                                                new PlainPoint { x = 850000, y = 7000000 },
                                                new PlainPoint { x = 800000, y = 6900000 }
                                        }
                                },
                                new PlainLinearRing { wrappedPoints = 
                                        new PlainPoint[]{
                                                new PlainPoint { x = 830000, y = 6910000 },
                                                new PlainPoint { x = 870000, y = 6910000 },
                                                new PlainPoint { x = 850000, y = 6950000 },
                                                new PlainPoint { x = 830000, y = 6910000 }
                                        }
                                }
                            }
                    }
            };
            // Step 8: Set some geometry options to alter the appearance of the shapes
            GeometryOption[] geometryOptions = new GeometryOption[]{
			    new GeometryOption { option = GeometryOptions.BORDERCOLOR, value = rgbToColorString(0,128,255)},
			    new GeometryOption { option = GeometryOptions.FILLCOLOR, value = rgbToColorString(255,0,0)},
			    new GeometryOption { option = GeometryOptions.FILLALPHA, value = "200"}, // 0-255
			    new GeometryOption { option = GeometryOptions.AUTOCENTEROBJECTS, value = "1"} // 0-255
		    };

            // Step 9: Create geometries from the shapes
            XMap.Geometry geometry1 = new XMap.Geometry
            {
                description = "firstPolygon",
                id = 0,
                geometry = polygon1,
                referencePoint = point
            };

            XMap.Geometry geometry2 = new XMap.Geometry
            {
                description = "secondPolygon",
                id = 1,
                geometry = polygon2,
                referencePoint = point
            };

            // Step 10: Bundle geometries with geometry options 
            XMap.Geometries[] geometries = new XMap.Geometries[] 
            {
                new XMap.Geometries {
                    wrappedGeometries = new XMap.Geometry[] { geometry1, geometry2 },
                    wrappedOptions = geometryOptions,
                }
            };

            // Step 11: Put the geometries into the specific geometry layer
            GeometryLayer layer = new GeometryLayer
            {
                name = "geomLayer",
                drawPriority = PRIORSTREET,
                visible = true,
                wrappedGeometries = geometries,
                objectInfos = ObjectInfoType.GEOMETRY
            };

            try
            {
                // Step 12: Return object map with the specific information
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, 
                        new Layer[] { layer }, // include custom layer in request
                        includeImageInResponse, null);

                // Step 13: Retrieve the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
        }
        
        private String rgbToColorString(int red, int green, int blue)
        {
            int color = (blue << 16) + (green << 8) + (red);
            return "" + color;
        }

        // draw priority constants, taken from the PTV xMap documentation
        public static  int PRIORMIN                  =0;       //Minimum priority
		public static  int PRIORBRIDGE               =10;      //Bridges
		public static  int PRIORHEIGHT               =20;      //Height
		public static  int PRIOROTHER                =30;      //Other
		public static  int PRIORRAIL                 =40;      //Rail
		public static  int PRIORTUNNEL               =50;      //Tunnels
		public static  int PRIORWATER                =60;      //Water
		public static  int PRIORBACKGROUND           =80;      //Background
		public static  int PRIORROUTE                =100;     //Routes
		public static  int PRIORTOLL                 =110;     //Toll data
		public static  int PRIORSCENICROUTE          =115;     //Scenic routes
		public static  int PRIORSTREET               =150;     //Streets
		public static  int PRIORTOWNPOINTBASE        =400;     //Town points
		public static  int PRIORONEWAYSIGN           =450;     //One-way symbol
		public static  int PRIORTOWNNAMEBASE         =500;     //City names
		public static  int PRIORTOWNNAME1            =501;     //City names category 14 (>500000)
		public static  int PRIORTOWNNAME2            =502;     //City names category 13 (250000-500000)
		public static  int PRIORTOWNNAME3            =503;     //City names category 12 (100000-250000)
		public static  int PRIORTOWNNAME4            =504;     //City names category 11 (50000-100000)
		public static  int PRIORTOWNNAME5            =505;     //City names category 10 (20000-50000)
		public static  int PRIORTOWNNAME6            =506;     //City names category 9 (10000-20000)
		public static  int PRIORTOWNNAME7            =507;     //City names category 8 (5000-10000)
		public static  int PRIORTOWNNAME8            =508;     //City names category 7 (3000-5000)
		public static  int PRIORTOWNNAME9            =509;     //City names category 6 (2000-3000)
		public static  int PRIORTOWNNAME10           =510;     //City names category 5 (1000-2000)
		public static  int PRIORTOWNNAME11           =511;     //City names category 4 (500-1000)
		public static  int PRIORTOWNNAME12           =512;     //City names category 3 (200-500)
		public static  int PRIORTOWNNAME13           =513;     //City names category 2 (100-200)
		public static  int PRIORTOWNNAME14           =514;     //City names category 1 (100)
		public static  int PRIORSTREETSIGNSBASE      =550;     //Street signs
		public static  int PRIORSTREETSIGNSMOTORWAY  =551;     //Street signs motorways
		public static  int PRIORSTREETSIGNSCOUNTRY   =552;     //Street signs trunk roads
		public static  int PRIORSTREETNAMEBASE       =600;     //Street names
		public static  int PRIORSTREETNAMEMOTORWAY   =601;     //Street names motorways
		public static  int PRIORSTREETNAMEEUROPE     =602;     //Street names Europe roads
		public static  int PRIORSTREETNAMENATIONAL   =603;     //Street names trunk roads
		public static  int PRIORSTREETNAMECOUNTRY    =604;     //Street names country roads
		public static  int PRIORSTREETNAMEREGION     =605;     //Street names county roads
		public static  int PRIORSTREETNAMETOWN       =606;     //Street names town streets
		public static  int PRIORSTREETNAMEZONE30     =607;     //Street names residential area streets
		public static  int PRIORSTREETNAMEPLAY       =608;     //Street names play streets
		public static  int PRIORADAKSIGN             =800;     //Signs of intersections
		public static  int PRIORHOUSENUMBER          =11000;    //House number
		public static  int PRIORTEXT                 =12000;   //General text
		public static  int PRIORPOIDEFAULT           =25000;   //Points of interest (on top)
		public static  int PRIORMAX                  =50000;   //Maximum priority
    }
}
