﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingWithPoints.XMap;

namespace MappingWithPoints
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample() {
		    // Step 1: Initialize xmap client
		    XMapWSService client = new XMapWSService();

		    // Step 2: Define mapSection and set center and scale
		    MapSection mapSection = new MapSection();
            XMap.Point point = new XMap.Point { point = new PlainPoint { x = 685407.751736, y = 6372537.965244 } };
		    mapSection.center = point;
		    mapSection.scale = 1000;

		    // Step 3: Create imageInfo and set relevant attributes
		    ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

		    // Step 4: Realize object mapParams showing scale without use of miles.
		    MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;

		    // Step 5: Define points for bitmaps
		    PlainPoint plainPoint1 = new PlainPoint { x = 681254.0, y = 6372399.0 };
		    XMap.Point point1 = new XMap.Point { point = plainPoint1};
		    PlainPoint plainPoint2 = new PlainPoint { x = 682875.0, y = 6369972.0};
		    XMap.Point point2 = new XMap.Point { point = plainPoint2 };

		    // Step 6: Fill bitmapArray with start and destination flag
		    Bitmap[] bitmapArray = new Bitmap[2];
		    bitmapArray[0] = new Bitmap{
                descr = "MyFirstBitmap",
                position = point1,
                name = "flaggreen.bmp"
            };
            bitmapArray[1] = new Bitmap{
                descr = "MySecondBitmap",
                position = point2,
                name = "flagred.bmp"
            };

		    // Step 7: Set bitmapOptions
		    BitmapOptions bitmapOptions = new BitmapOptions();
		    // change the reference point for the pixel position of the bitmap
		    // by default the bitmap center will be used.
		    // here: bitmap displacement of 20 pixels.
		    PlainPoint referencePoint = new PlainPoint { x = 20, y = 20 };
		    bitmapOptions.referencePoint = referencePoint;
		    bitmapOptions.transparencyColor = new XMap.Color { red  = 255, green = 255, blue = 255 };

		    // Step 8: Add bitmapArray and bitmapOptions into bitmaps
		    Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps { wrappedBitmaps = bitmapArray, options = bitmapOptions } };

		    // Step 9: Initialize customLayer with the relevant settings
		    CustomLayer customLayer = new CustomLayer {
                centerObjects = true,
                drawPriority = 25000,
                name = "ObjectLayer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps
            };

		    // Step 10: Put customLayer into layers
		    Layer[] layers = new Layer[] { customLayer };

		    // Step 11: Define boolean value including image in response
		    bool includeImageInResponse = false;
		
		    try {
			    // Step 12: Return map object with its characteristics
			    Map map = client.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse, null);
			    ObjectInfos[] objInfo = map.wrappedObjects;

			    // Step 14: Retrieve image
			    String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

		    } catch (Exception e) {
			    // proper exception handling: see API for PTV's exception handling!
		    }
	    }
    }
}
