﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingWithPointsAndLines.XMap;

namespace MappingWithPointsAndLines
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: Initialize xmap client
            XMapWSService xMapClient = new XMapWSService();
            // Step 2: define start and destination points as type of PlainPoint
            PlainPoint startPoint = new PlainPoint { x = 681602.0, y = 6372219.0 };
            double[][] pointCoordinates = new double[][] { new double[] { 680361.0, 6370092.0 }, new double[] { 680868.0, 6369835.0 },
				new double[] { 683232.0, 6374948.0 }, new double[] { 679510.0, 6373244.0 }, new double[] { 682756.0, 6368686.0 }, new double[] { 684163.0, 6372005.0 },
				new double[] { 682071.0, 6376094.0 }, new double[] { 679305.0, 6371392.0 }, };
            int numberOfDestinationPoints = pointCoordinates.Length;

            PlainPoint[] destinationPlainPoints = new PlainPoint[numberOfDestinationPoints];
            for (int i = 0; i < numberOfDestinationPoints; i++)
            {
                destinationPlainPoints[i] = new PlainPoint();
                destinationPlainPoints[i].x = pointCoordinates[i][0];
                destinationPlainPoints[i].y = pointCoordinates[i][1];
            }

            // Step 2.1: set color values for each line
            int[][] colorValues = new int[][] {
				// {blue, green, red}
				new int[] { 255, 0, 0 }, new int[] { 0, 0, 255 }, new int[] { 0, 255, 0 }, new int[] { 238, 134, 28 }, 
                new int[] { 139, 123, 108 }, new int[] { 79, 79, 47 }, new int[] { 131, 139, 131 }, new int[] { 0, 290, 139 } };

            // Step 3: define an array of lines
            Lines[] lines = new Lines[numberOfDestinationPoints];

            for (int i = 0; i < numberOfDestinationPoints; i++)
            {
                // Step 3.1: create plainLineString
                PlainLineString plainLineString = new PlainLineString();
                plainLineString.wrappedPoints = new PlainPoint[] { startPoint, destinationPlainPoints[i] };
                // Step 3.2: set object into lineStrings[0]. Note: object lineStrings has to be reinitialized in every loop.
                LineString[] lineStrings = new LineString[1];
                lineStrings[0] = new LineString();
                lineStrings[0].lineString = plainLineString;
                // Step 3.3: create new lines[i] and fill array
                lines[i] = new Lines();
                lines[i].wrappedLines = lineStrings;
                // Step 3.4: set lineOptions transparency
                LineOptions lineOptions = new LineOptions();
                lineOptions.transparent = true;
                // Step 3.5: set linePartOptions visible, define width of line and
                // specific color
                LinePartOptions linePartOptions = new LinePartOptions();
                linePartOptions.visible = true;
                linePartOptions.width = 7;
                linePartOptions.color = new XMap.Color { red = colorValues[i][0], green = colorValues[i][1], blue = colorValues[i][2] };
                // Step 3.6: set main line
                lineOptions.mainLine = linePartOptions;
                // Step 3.7: set all options for specific line
                lines[i].options = lineOptions;
            }

            // Step 4: generate a new object named arrBitmap as type of Bitmap[]
            Bitmap[] arrBitmap = new Bitmap[numberOfDestinationPoints + 1];
            for (int i = 0; i < numberOfDestinationPoints; i++)
            {
                XMap.Point bitmapPointDestination = new XMap.Point();
                bitmapPointDestination.point = destinationPlainPoints[i];
                String description = (i + 2) + ". bitmap: destination";
                // Step 4.1: set destination point, description and select the
                // corresponding flag
                arrBitmap[i] = new Bitmap
                {
                    descr = description,
                    name = "flagred.bmp",
                    position = bitmapPointDestination
                };
            }

            XMap.Point bitmapPointStart = new XMap.Point();
            bitmapPointStart.point = startPoint;
            // Step 4.2: set start point, description and select the corresponding flag
            arrBitmap[8] = new Bitmap
            {
                descr = "1. bitmap: start",
                name = "location.bmp",
                position = bitmapPointStart
            };

            // Step 5: create bitmapOptions and set color for transparency
            BitmapOptions bitmapOptions = new BitmapOptions();
            bitmapOptions.transparencyColor = new XMap.Color { red = 255, green = 255, blue = 255 };

            // Step 6: create bitmaps and put arrBitmap and bitmapOptions into this
            // object
            Bitmaps[] bitmaps = new Bitmaps[1];
            bitmaps[0] = new Bitmaps
            {
                wrappedBitmaps = arrBitmap,
                options = bitmapOptions
            };

            // Step 7: define customLayer and set the relevant attributes
            CustomLayer customLayer = new CustomLayer
            {
                drawPriority = 25000,
                name = "Text layer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps,
                wrappedLines = lines
            };
            // Step 7.1: add customLayer into layers
            Layer[] layers = new Layer[] { customLayer };

            // Step 8: define mapSection and set center and scale
            MapSection mapSection = new MapSection();
            XMap.Point point = new XMap.Point();
            point.point = startPoint;
            mapSection.center = point;
            mapSection.scale = 400;

            // Step 9: create object imageInfo, define the relevant attributes
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 10: set values as specified
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;

            // Step 11: set value to false.
            bool includeImageInResponse = false;

            try
            {
                // Step 12: return object map with the comprehensive information
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 13: retrieve the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // TODO proper exception handling
            }
        }
    }
}
