﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingWithTruckAttributes.XMap;

namespace MappingWithTruckAttributes
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: Initialize xmap client
            XMapWSService xMapClient = new XMapWSService();
            // Step 2: Create CallerContext and use profile "roadattributes"
            // create object CallerContext
            CallerContext cxt = new CallerContext();
            cxt.log1 = "Mapping with Truck Attributes";
            // set CallerContext properties to define profile and CoordFormat
            CallerContextProperty props1 = new CallerContextProperty
            {
                key = "Profile",
                value = "default"
            };
            CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
            CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
            cxt.wrappedProperties = properties;

            // Step 3: Define class mapSection
            MapSection mapSection = new MapSection();
            XMap.Point point = new XMap.Point();
            point.point = new PlainPoint { x = 681490.0, y = 6372500.0 };
            mapSection.center = point;
            mapSection.scale = 150;

            // Step 4: Set imageInfo
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 5: Specify mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;
            mapParams.language = "x-ptv-DFT";

            // Step 6: Set value to false
            bool includeImageInResponse = false;

            // ######################################################################################
            // Copy the map-dependent files such as truck_attributes_<version>.bin into
            // the "truckattributes" folder which should be added into the
            // <root-directory>/re folder before.
            // Otherwise an xMap exception will be thrown!
            // ######################################################################################

            // Step 7: Add layers
            RoadEditorLayer reLayer = new RoadEditorLayer();
            // Demo version: maps.path=data/map is enabled.
            // re/truckattributes folder is already available --> see folder data/map/re
            reLayer.name = "truckattributes";
            reLayer.visible = true;
            reLayer.objectInfos = ObjectInfoType.NONE;
            Layer[] layers = new Layer[] { reLayer };

            try
            {
                // Step 8: Call method renderMap
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse, cxt);

                // Step 9: Get the image from object map
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
        }
    }
}
