﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MatchingBasic.XMapMatch;

namespace MatchingBasic
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            System.Threading.Thread.CurrentThread.CurrentCulture = new System.Globalization.CultureInfo("en-IN");
            System.Threading.Thread.CurrentThread.CurrentUICulture = new System.Globalization.CultureInfo("en-IN");
            // Call MapMatchingBasic
            MatchResult matchTrack = new MatchResult();
            try
            {
                runMapMatching(ref matchTrack);
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }

            // create result table MapMatchResult
            List<TableEntryMapMathchingResult[]> MapMatchResult = new List<TableEntryMapMathchingResult[]>();
            runMapMatchingResult(matchTrack, ref MapMatchResult);

            // create result table MapCompletePath
            List<List<TableEntryMapMathchingCompletePath>> MapCompletePath = new List<List<TableEntryMapMathchingCompletePath>>();
            runMapMatchingForCompletePath(matchTrack, ref MapCompletePath);

            mmRes.ItemsSource = MapMatchResult[0];
            mmComp.ItemsSource = MapCompletePath[0]; 
        }
        private static void runMapMatching(ref MatchResult matchTrack)
        {

		    // Step 1: Initialize xmap client
            XMapmatchWSService xMapMatchClient = new XMapmatchWSService();

		    // step 2:
		    // load GPS-track
		    List<String> gps = loadTrack();

		    // step 3:
		    // create object CallerContext
		    CallerContext cxt = new CallerContext();
		    cxt.log1 = "Basic Map Matching";
		    // set CallerContext properties to define profile and CoordFormat
		    CallerContextProperty props1 = new CallerContextProperty();
		    props1.key = "Profile";
		    props1.value = "default";
		    CallerContextProperty props2 = new CallerContextProperty();
		    props2.key = "CoordFormat";
		    props2.value = "OG_GEODECIMAL";
		    CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
		    cxt.wrappedProperties = properties;

		    // step 4:
		    // create object trackPositions
		    int max = gps.Count;
		    TrackPosition[] trackPositions = new TrackPosition[max];

		    for (int i = 0; i < gps.Count; i++) 
            {
			    String arg0 = gps[i];
                String[] trackData = arg0.Split(';');
                TrackPosition trackPosition = new TrackPosition();

			    int id = Convert.ToInt32(trackData[0]);
			    trackPosition.id = id;
			    
                float heading = Convert.ToSingle(trackData[1]);
			    trackPosition.heading = heading;
			   
                float speedInMps = Convert.ToSingle(trackData[2]);
			    trackPosition.speedInMps = speedInMps;
			    
                System.DateTime timestamp = Convert.ToDateTime(trackData[3]);
			    trackPosition.timestamp = timestamp;
			    
                double x = Convert.ToDouble(trackData[4]);
			    trackPosition.lon = x;
			    
                double y = Convert.ToDouble(trackData[5]);
			    trackPosition.lat = y;
                
                trackPosition.latSpecified = true;
                trackPosition.lonSpecified = true;
                trackPosition.timestampSpecified = true;

			    trackPositions[i] = trackPosition;
		    }

		    try {
			    // step 5: execute matchTrack
			    //matchTrack = xMapMatchClient.matchTrack(trackPositions, "lux_d", cxt);
                matchTrack = xMapMatchClient.matchTrackExtended(trackPositions, "lux_d", true, true, cxt);
		    } 
            catch (Exception e)
            {
                //TODO: proper exception handling
		    }
	    }

	    private static List<String> loadTrack() {
		    List<String> h = new List<String>();
		    // Use OG_Geodecimal
		    h.Add("1;251.065;5.00004;2011-07-06T07:57:03.000+0200;6.114670;49.626195");
		    h.Add("2;251.065;5.00004;2011-07-06T07:57:04.000+0200;6.114605;49.626182");
		    h.Add("3;251.065;5.00004;2011-07-06T07:57:05.000+0200;6.114538;49.626167");
		    h.Add("4;251.065;5.00004;2011-07-06T07:57:06.000+0200;6.114473;49.626152");
		    h.Add("5;251.065;5.00004;2011-07-06T07:57:07.000+0200;6.114408;49.626137");
		    h.Add("6;251.065;5.00004;2011-07-06T07:57:08.000+0200;6.114342;49.626123");
		    h.Add("7;251.065;5.00004;2011-07-06T07:57:09.000+0200;6.114277;49.626108");
		    h.Add("8;251.065;5.00004;2011-07-06T07:57:10.000+0200;6.114210;49.626093");
		    h.Add("9;251.065;5.00004;2011-07-06T07:57:11.000+0200;6.114145;49.626078");
		    h.Add("10;251.065;5.00004;2011-07-06T07:57:12.000+0200;6.114080;49.626065");
		    h.Add("11;251.065;5.00004;2011-07-06T07:57:13.000+0200;6.114013;49.626050");
		    h.Add("12;251.065;5.00004;2011-07-06T07:57:14.000+0200;6.113948;49.626065");
		    h.Add("13;224.177;5.00004;2011-07-06T07:57:15.000+0200;6.113885;49.626018");
		    h.Add("14;224.177;5.00004;2011-07-06T07:57:16.000+0200;6.113837;49.625987");
		    h.Add("15;224.177;5.00004;2011-07-06T07:57:17.000+0200;6.113788;49.625953");
		    h.Add("16;162.053;5.00004;2011-07-06T07:57:18.000+0200;6.113772;49.625917");
		    h.Add("17;162.053;5.00004;2011-07-06T07:57:19.000+0200;6.113793;49.625873");
		    h.Add("18;162.053;5.00004;2011-07-06T07:57:20.000+0200;6.113815;49.625832");
		    h.Add("19;162.053;5.00004;2011-07-06T07:57:21.000+0200;6.113837;49.625788");
		    h.Add("20;162.053;5.00004;2011-07-06T07:57:22.000+0200;6.113857;49.625745");
		    h.Add("21;162.053;5.00004;2011-07-06T07:57:23.000+0200;6.113878;49.625703");
		    h.Add("22;162.053;5.00004;2011-07-06T07:57:24.000+0200;6.113900;49.625660");
		    h.Add("23;162.053;5.00004;2011-07-06T07:57:25.000+0200;6.113922;49.625612");
		    h.Add("24;162.053;5.00004;2011-07-06T07:57:26.000+0200;6.113943;49.625575");
		    h.Add("25;162.053;5.00004;2011-07-06T07:57:27.000+0200;6.113965;49.625532");
		    h.Add("26;162.053;5.00004;2011-07-06T07:57:28.000+0200;6.113985;49.625549");
		    h.Add("27;162.053;5.00004;2011-07-06T07:57:29.000+0200;6.114007;49.625447");
		    h.Add("28;147.066;5.00004;2011-07-06T07:57:30.000+0200;6.114033;49.625405");
		    h.Add("29;147.066;5.00004;2011-07-06T07:57:31.000+0200;6.114072;49.625367");
		    h.Add("30;147.066;5.00004;2011-07-06T07:57:32.000+0200;6.114110;49.625330");
		    h.Add("31;147.066;5.00004;2011-07-06T07:57:33.000+0200;6.114147;49.625292");
		    h.Add("32;147.066;5.00004;2011-07-06T07:57:34.000+0200;6.114185;49.625253");
		    h.Add("33;150.959;5.00004;2011-07-06T07:57:35.000+0200;6.114220;49.625215");
		    h.Add("34;150.959;5.00004;2011-07-06T07:57:36.000+0200;6.114253;49.625175");
		    h.Add("35;150.959;5.00004;2011-07-06T07:57:37.000+0200;6.114287;49.625137");
		    h.Add("36;150.959;5.00004;2011-07-06T07:57:38.000+0200;6.114320;49.625097");
		    h.Add("37;150.959;5.00004;2011-07-06T07:57:39.000+0200;6.114355;49.625058");
		    h.Add("38;150.959;5.00004;2011-07-06T07:57:40.000+0200;6.114388;49.625018");
		    h.Add("39;150.959;5.00004;2011-07-06T07:57:41.000+0200;6.114422;49.624978");
		    h.Add("40;148.972;5.00004;2011-07-06T07:57:42.000+0200;6.114455;49.624940");
		    h.Add("41;148.972;5.00004;2011-07-06T07:57:43.000+0200;6.114492;49.624902");
		    h.Add("42;148.972;5.00004;2011-07-06T07:57:44.000+0200;6.114527;49.624863");
		    h.Add("43;148.972;5.00004;2011-07-06T07:57:45.000+0200;6.114563;49.624825");
		    h.Add("44;148.972;5.00004;2011-07-06T07:57:46.000+0200;6.114598;49.624785");
		    h.Add("45;148.972;5.00004;2011-07-06T07:57:47.000+0200;6.114635;49.624747");
		    h.Add("46;148.972;5.00004;2011-07-06T07:57:48.000+0200;6.114671;49.624708");
		    h.Add("47;148.972;5.00004;2011-07-06T07:57:49.000+0200;6.114705;49.624670");
		    h.Add("48;148.972;5.00004;2011-07-06T07:57:50.000+0200;6.114742;49.624632");
		    h.Add("49;148.972;5.00004;2011-07-06T07:57:51.000+0200;6.114777;49.624593");
		    h.Add("50;148.972;5.00004;2011-07-06T07:57:52.000+0200;6.114813;49.624555");
		    h.Add("51;148.972;5.00004;2011-07-06T07:57:53.000+0200;6.114848;49.624517");
		    h.Add("52;148.972;5.00004;2011-07-06T07:57:54.000+0200;6.114885;49.624477");
		    h.Add("53;148.972;5.00004;2011-07-06T07:57:55.000+0200;6.114920;49.624438");
		    h.Add("54;148.972;5.00004;2011-07-06T07:57:56.000+0200;6.114957;49.624400");
		    h.Add("55;148.972;5.00004;2011-07-06T07:57:57.000+0200;6.114992;49.624362");
		    h.Add("56;154.087;5.00004;2011-07-06T07:57:58.000+0200;6.115027;49.624323");
		    h.Add("57;154.087;5.00004;2011-07-06T07:57:59.000+0200;6.115057;49.624282");
		    h.Add("58;154.087;5.00004;2011-07-06T07:58:00.000+0200;6.115087;49.624242");
		    h.Add("59;154.087;5.00004;2011-07-06T07:58:01.000+0200;6.115117;49.624202");
		    h.Add("60;154.087;5.00004;2011-07-06T07:58:02.000+0200;6.115147;49.624162");
		    h.Add("61;154.087;5.00004;2011-07-06T07:58:03.000+0200;6.115178;49.624120");
		    h.Add("62;154.087;5.00004;2011-07-06T07:58:04.000+0200;6.115208;49.624080");
		    h.Add("63;154.087;5.00004;2011-07-06T07:58:05.000+0200;6.115238;49.624040");
		    h.Add("64;154.087;5.00004;2011-07-06T07:58:06.000+0200;6.115268;49.623998");
		    h.Add("65;142.919;5.00004;2011-07-06T07:58:07.000+0200;6.115308;49.623962");
		    h.Add("66;142.919;5.00004;2011-07-06T07:58:08.000+0200;6.115350;49.623927");
		    h.Add("67;142.919;5.00004;2011-07-06T07:58:09.000+0200;6.115392;49.623890");
		    h.Add("68;142.919;5.00004;2011-07-06T07:58:10.000+0200;6.115433;49.623855");
		    h.Add("69;142.919;5.00004;2011-07-06T07:58:11.000+0200;6.115475;49.623818");
		    h.Add("70;142.919;5.00004;2011-07-06T07:58:12.000+0200;6.115517;49.623783");
		    h.Add("71;142.919;5.00004;2011-07-06T07:58:13.000+0200;6.115558;49.623747");
		    h.Add("72;142.919;5.00004;2011-07-06T07:58:14.000+0200;6.115602;49.623712");
		    h.Add("73;142.919;5.00004;2011-07-06T07:58:15.000+0200;6.115643;49.623675");
		    h.Add("74;142.919;5.00004;2011-07-06T07:58:16.000+0200;6.115685;49.623640");
		    h.Add("75;142.919;5.00004;2011-07-06T07:58:17.000+0200;6.115727;49.623603");
		    h.Add("76;142.919;5.00004;2011-07-06T07:58:18.000+0200;6.115768;49.623568");
		    h.Add("77;142.919;5.00004;2011-07-06T07:58:19.000+0200;6.115810;49.623532");
		    h.Add("78;142.919;5.00004;2011-07-06T07:58:20.000+0200;6.115852;49.623495");
		    h.Add("79;144.254;5.00004;2011-07-06T07:58:21.000+0200;6.115893;49.623460");
		    h.Add("80;144.254;5.00004;2011-07-06T07:58:22.000+0200;6.115933;49.623423");
		    h.Add("81;144.254;5.00004;2011-07-06T07:58:23.000+0200;6.115975;49.623387");
		    h.Add("82;144.254;5.00004;2011-07-06T07:58:24.000+0200;6.116015;49.623350");
		    h.Add("83;144.254;5.00004;2011-07-06T07:58:25.000+0200;6.116055;49.623313");
		    h.Add("84;144.254;5.00004;2011-07-06T07:58:26.000+0200;6.116097;49.623277");
		    h.Add("85;144.254;5.00004;2011-07-06T07:58:27.000+0200;6.116137;49.623240");
		    h.Add("86;144.254;5.00004;2011-07-06T07:58:28.000+0200;6.116177;49.623205");
		    h.Add("87;144.254;5.00004;2011-07-06T07:58:29.000+0200;6.116218;49.623168");
		    h.Add("88;144.254;5.00004;2011-07-06T07:58:30.000+0200;6.116258;49.623132");
		    h.Add("89;144.254;5.00004;2011-07-06T07:58:31.000+0200;6.116298;49.623095");
		    h.Add("90;144.526;5.00004;2011-07-06T07:58:32.000+0200;6.116340;49.623058");
		    h.Add("91;144.526;5.00004;2011-07-06T07:58:33.000+0200;6.116380;49.623022");
		    h.Add("92;144.526;5.00004;2011-07-06T07:58:34.000+0200;6.116420;49.622985");
		    h.Add("93;144.526;5.00004;2011-07-06T07:58:35.000+0200;6.116460;49.622948");
		    h.Add("94;144.526;5.00004;2011-07-06T07:58:36.000+0200;6.116500;49.622912");
		    h.Add("95;144.526;5.00004;2011-07-06T07:58:37.000+0200;6.116540;49.622875");
		    h.Add("96;144.526;5.00004;2011-07-06T07:58:38.000+0200;6.116582;49.622838");
		    h.Add("97;144.526;5.00004;2011-07-06T07:58:39.000+0200;6.116662;49.622765");
		    h.Add("98;144.526;5.00004;2011-07-06T07:58:40.000+0200;6.116702;49.622728");
		    h.Add("99;144.526;5.00004;2011-07-06T07:58:41.000+0200;6.116742;49.622692");
		    h.Add("100;144.526;5.00004;2011-07-06T07:58:42.000+0200;6.116782;49.622655");
		    h.Add("101;144.526;5.00004;2011-07-06T07:58:43.000+0200;6.116815;49.622615");
		    h.Add("102;154.086;5.00004;2011-07-06T07:58:44.000+0200;6.116845;49.622575");
		    h.Add("103;154.086;5.00004;2011-07-06T07:58:45.000+0200;6.116875;49.622535");
		    h.Add("104;154.086;5.00004;2011-07-06T07:58:46.000+0200;6.116905;49.622495");
		    return h;
	    }

        private static void runMapMatchingResult(MatchResult matchTrack, ref List<TableEntryMapMathchingResult[]> MapMatchingTable) 
        {
		    int index = 0;
		    int maxMatchedLocations = matchTrack.wrappedMatchedLocations.Length;

            TableEntryMapMathchingResult[] rows = new TableEntryMapMathchingResult[maxMatchedLocations];

		    foreach (MatchedLocation matchedLocation in matchTrack.wrappedMatchedLocations) {
                String lat = "" + String.Format("{0}", matchedLocation.lat);
			    String lon = "" + String.Format("{0}", matchedLocation.lon);
                String tileID = "";
                foreach (MatchedSegment matchedSegment in matchedLocation.wrappedPath)
                {
                    tileID = tileID + matchedSegment.tileId + ";";
                }

                String networkClass = "";
                foreach (MatchedSegment matchedSegment in matchedLocation.wrappedPath)
                {
                    networkClass = networkClass + matchedSegment.networkClass + ";";
                }
                // create table entry 
                TableEntryMapMathchingResult row = new TableEntryMapMathchingResult
                {
                    AngleDiff = String.Format("{0}",
					    matchedLocation.angleDifference),
                    CovDist = String.Format("{0}",
					    matchedLocation.coveredDistance),
                    DrivenDist = String.Format("{0}",
					    matchedLocation.drivenDistance), 
                    Heading = String.Format("{0}", 
                        matchedLocation.heading),
                    LinkingDist = String.Format("{0}",
					    matchedLocation.linkingDistance),
                    LocalMatching = matchedLocation.localMatching,
                    LocalRating = String.Format("{0}",
					    matchedLocation.localRating),
                    Prob = String.Format("{0}",
					    matchedLocation.probability * 100),
                    Stable = matchedLocation.stable,
                    Timestamp = matchedLocation.timestamp,
                    TransRating = String.Format("{0}",
					    matchedLocation.transitionRating), 
                    InputID = matchedLocation.inputId,
                    Pos = lat + ";" + lon,
                    TileID = tileID,
                    NetClass  = networkClass
                };
                rows[index] = row;
                index++;
		    }
            MapMatchingTable.Add(rows);
	    }

        private static void runMapMatchingForCompletePath(MatchResult matchTrack, ref List<List<TableEntryMapMathchingCompletePath>> MapMatchingTable) 
        {
		    int index = 0;
		    int maxCompletePath = matchTrack.wrappedCompletePath.Length;

            List<TableEntryMapMathchingCompletePath> rows = new List<TableEntryMapMathchingCompletePath>();

            foreach (MatchedLocation matchedLocation in matchTrack.wrappedMatchedLocations)
            {
                foreach (MatchedSegment matchedSegment in matchedLocation.wrappedPath)
                {
                    String endXYN = "";
                    foreach (int i in matchedSegment.wrappedEndXYN)
                    {
                        endXYN = endXYN + i + ";";
                    }
                    String startXYN = "";
                    foreach (int i in matchedSegment.wrappedStartXYN)
                    {
                        startXYN = startXYN + i + ";";
                    }
                    // create table entry 
                    TableEntryMapMathchingCompletePath row = new TableEntryMapMathchingCompletePath()
                    {
                        tileID = matchedSegment.tileId,
                        rtgSegIdx = matchedSegment.rtgSegIdx,
                        networkClass = matchedSegment.networkClass,
                        speedLimitBackward = matchedSegment.speedLimitBackward,
                        speedLimitForward = matchedSegment.speedLimitForward,
                        length = matchedSegment.length,
                        countryCode = matchedSegment.countryCode,
                        endXYN = endXYN,
                        startXYN = startXYN
                    };
                    rows.Add(row);
                    index++;
                }
		    }
		    MapMatchingTable.Add(rows);
	    }

        // rename aut-generated column headers
        private void mmRes_AutoGeneratingColumn(object sender, DataGridAutoGeneratingColumnEventArgs e)
        {
            if (e.Column.Header.ToString() == "AngleDiff")
                e.Column.Header = "Angle Difference [°]";
            if (e.Column.Header.ToString() == "CovDist")
                e.Column.Header = "Covered Distance [m]";
            if (e.Column.Header.ToString() == "DrivenDist")
                e.Column.Header = "Driven distance [m]";
            if (e.Column.Header.ToString() == "LinkingDist")
                e.Column.Header = "Heading [°]";
            if (e.Column.Header.ToString() == "LinkingDist")
                e.Column.Header = "Linking Distance [m]";
            if (e.Column.Header.ToString() == "LocalMatching")
                e.Column.Header = "Local Matching";
            if (e.Column.Header.ToString() == "LocalRating")
                e.Column.Header = "Local Rating";
            if (e.Column.Header.ToString() == "Prob")
                e.Column.Header = "Probability [%]";
            if (e.Column.Header.ToString() == "Stable")
                e.Column.Header = "Stable";
            if (e.Column.Header.ToString() == "Timestamp")
                e.Column.Header = "Timestamp";
            if (e.Column.Header.ToString() == "TransRating")
                e.Column.Header = "Transition Rating";
            if (e.Column.Header.ToString() == "InputID")
                e.Column.Header = "Input Id";
            if (e.Column.Header.ToString() == "Pos")
                e.Column.Header = "Position [lat;lon]";
            if (e.Column.Header.ToString() == "TileID")
                e.Column.Header = "Tile Id";
            if (e.Column.Header.ToString() == "NetClass")
                e.Column.Header = "Network Class";
        }

        // rename aut-generated column headers
        private void mmComp_AutoGeneratingColumn(object sender, DataGridAutoGeneratingColumnEventArgs e)
        {
            if (e.Column.Header.ToString() == "tileID")
                e.Column.Header = "Tile Id";
            if (e.Column.Header.ToString() == "rtgSegIdx")
                e.Column.Header = "rtgSegIdx";
            if (e.Column.Header.ToString() == "networkClass")
                e.Column.Header = "Network Class";
            if (e.Column.Header.ToString() == "speedLimitBackward")
                e.Column.Header = "Speed Limit Backward";
            if (e.Column.Header.ToString() == "speedLimitForward")
                e.Column.Header = "Speed Limit Forward";
            if (e.Column.Header.ToString() == "length")
                e.Column.Header = "Length [m]";
            if (e.Column.Header.ToString() == "countryCode")
                e.Column.Header = "Country Code";
            if (e.Column.Header.ToString() == "endXYN")
                e.Column.Header = "End XYN";
            if (e.Column.Header.ToString() == "startXYN")
                e.Column.Header = "Start XYN";
        }
    }

    // class for row entries in Map Matching Result table
    class TableEntryMapMathchingResult
    {
        public String AngleDiff { get; set; }
        public String CovDist { get; set; }
        public String DrivenDist { get; set; }
        public String Heading { get; set; }
        public String LinkingDist { get; set; }
        public bool LocalMatching { get; set; }
        public String LocalRating { get; set; }
        public String Prob { get; set; }
        public bool Stable { get; set; }
        public DateTime Timestamp { get; set; }
        public String TransRating { get; set; }
        public long InputID { get; set; }
        public String Pos { get; set; }
        public String TileID { get; set; }
        public String NetClass { get; set; }
    }

    // class for row entries in Map Matching Complete Path table
    class TableEntryMapMathchingCompletePath
    {
        public long tileID { get; set; }
        public int rtgSegIdx { get; set; }
        public int networkClass { get; set; }
        public int speedLimitBackward { get; set; }
        public int speedLimitForward { get; set; }
        public int length { get; set; }
        public int countryCode { get; set; }
        public String endXYN { get; set; }
        public String startXYN { get; set; }
    }
}
