﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RestrictionZones
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // calculate route
            Route r = calculateRoute();

            // display route
            if (r != null)
                displayMapOfRoute(r);
        }

        private Route calculateRoute()
        {
            // Step 1: Initialize client
            XRouteWSService xRouteClient = new XRouteWSService();

            // Step 2.1: Initialize CallerContext and add properties
            CallerContext callerContext = new CallerContext();
            CallerContextProperty profile = new CallerContextProperty { key = "Profile", value = "truckfast" };
            CallerContextProperty format = new CallerContextProperty { key = "CoordFormat", value = "PTV_GEODECIMAL" };

            // Step 2.2: If RestrictionZones are used add an XML Snippet to the CallerContext
            CallerContextProperty useRestrictionZones = new CallerContextProperty();
            if (useRestriction.IsChecked == true)
            {
                String xml = @"<?xml version=""1.0"" encoding=""UTF-8"" standalone=""yes""?>
                            <Profile xmlns:ns2=""http://www.ptv.de/com/ptvag/xserver/engines/xpoidbaccess"" >
                                <FeatureLayer majorVersion=""1"" minorVersion=""0"">
                                    <Themes>
                                        <Theme id=""PTV_RestrictionZones"" enabled=""true"">
                                        </Theme>
                                    </Themes>
                                </FeatureLayer>
                                <Routing majorVersion=""2"" minorVersion=""0"">
                                    <Course>
                                        <AdditionalDataRules enabled=""true""/>
                                    </Course>
                                </Routing>
                            </Profile>";
                useRestrictionZones = new CallerContextProperty();
                useRestrictionZones.key = "ProfileXMLSnippet";
                useRestrictionZones.value = xml;
            }

            callerContext.wrappedProperties = new CallerContextProperty[] { profile, useRestrictionZones, format };

            // Step 3: Specify the ResultListOptions
            ResultListOptions resultListOptions = new ResultListOptions
            {
                polygon = true,
                totalRectangle = true,
                detailLevel = DetailLevel.STANDARD,
                segments = true,
                segmentAttributes = true,
                
            };

            // Step 4: Get array waypointsDesc[]
            WaypointDesc[] waypointDescs = getWaypoints();

            try
            {
                // Step 5: Call method calculateRoute
                Route route = xRouteClient.calculateRoute(waypointDescs, null, null, resultListOptions, callerContext);

                return route;
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
            return null;
        }

        private void displayMapOfRoute(Route route)
        {
            // Step 1: Initialize client
            XMapWSService xMapClient = new XMapWSService();

            // Step 2.1: Initialize CallerContext and add properties
            CallerContext callerContext = new CallerContext();
            CallerContextProperty format = new CallerContextProperty { key = "CoordFormat", value = "PTV_GEODECIMAL" };         

            // Step 2.2: If RestrictionZones are used add an XML Snippet to the CallerContext
            CallerContextProperty useRestrictionZones = new CallerContextProperty();
            if (useRestriction.IsChecked == true)
            {
                String xml = @"<?xml version=""1.0"" encoding=""UTF-8"" standalone=""yes""?>
                            <Profile xmlns:ns2=""http://www.ptv.de/com/ptvag/xserver/engines/xpoidbaccess"" >
                                <FeatureLayer majorVersion=""1"" minorVersion=""0"">
                                    <Themes>
                                        <Theme id=""PTV_RestrictionZones"" enabled=""true"">
                                        </Theme>
                                    </Themes>
                                </FeatureLayer>
                                <Rendering majorVersion=""1"" minorVersion=""0"">
                                    <Country id=""49"">
                                        <Theme id=""PTV_RestrictionZones"">
                                            <!-- specifies the base profile found in default.xml in your xmap directory -->
                                            <Class name=""TruckTransitZones"">
                                                <Style>
                                                    <!-- display PTV_RestrictionZones in turqois for Germany -->
                                                    <Geometry displayRange=""9-14"">
                                                        <RenderLevel renderOffset=""+10""/>
                                                        <Stroke color=""#2DAEC4""/>
                                                    </Geometry>
                                                    <Geometry displayRange=""15-23"">
                                                        <RenderLevel renderOffset=""+10""/>
                                                        <Stroke color=""#2DAEC4""/>
                                                    </Geometry>
                                                </Style>
                                            </Class>
                                        </Theme>
                                    </Country>
                                </Rendering>
                            </Profile>";
                useRestrictionZones = new CallerContextProperty();
                useRestrictionZones.key = "ProfileXMLSnippet";
                useRestrictionZones.value = xml;
            }

            callerContext.wrappedProperties = new CallerContextProperty[] { useRestrictionZones, format };

            // Step 3: Define imageInfo
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 4: Add mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;
            mapParams.language = "x-ptv-DFT"; //optional
            bool includeImageInResponse = false;

            // Step 5: Create a line from the given route
            LineString[] lineStrings = new LineString[] { route.polygon };
            Lines[] lines = new Lines[1];
            LineOptions options = new LineOptions();
            LinePartOptions linePartOpt = new LinePartOptions
            {
                color = new xserver.Color { red = 0, blue = 255, green = 0 },
                visible = true,
                width = -12
            };
            options.mainLine = linePartOpt;

            lines[0] = new Lines { wrappedLines = lineStrings, options = options };

            // Step 6: Define a custom layer that contains the line(s)
            CustomLayer customLayer = new CustomLayer
            {
                visible = true,
                drawPriority = 100, // this is PRIORROUTE
                wrappedLines = lines,
                objectInfos = ObjectInfoType.NONE,
                centerObjects = true
            };

            Layer[] layers = new Layer[] {};
            layers = new Layer[] { customLayer };
            
            // Step 7: Set the bounding box
            BoundingBox boundingBox = new BoundingBox
            {
                leftTop = route.totalRectangle.leftBottom,
                rightBottom = route.totalRectangle.rightTop,
            };

            try
            {
                // Step 8: Call method renderMapBoundingBox
                Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse, callerContext);
                // Step 9: retrieve and display the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
        }

        private WaypointDesc[] getWaypoints()
        {
            // points = list of pairs of (x,y) coordinates (in order of route)
            double[][] points = new double[][] { new double[] { 1153221, 4814020 }, new double[] { 1163108, 4813848 } };

            WaypointDesc[] waypointDescs = new WaypointDesc[points.Length];
            for (int i = 0; i < points.Length; i++)
            {
                xserver.Point pt = new xserver.Point();
                pt.point = new PlainPoint { x = points[i][0], y = points[i][1] };

                waypointDescs[i] = new WaypointDesc
                {
                    wrappedCoords = new xserver.Point[] { pt },
                    linkType = LinkType.NEXT_SEGMENT
                };
            }
            return waypointDescs;
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            runSample();
        }
    }
}
