﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingAvoidCountries
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: Initialize clients
            XMapWSService xMapClient = new XMapWSService();
            XRouteWSService xRouteClient = new XRouteWSService();

            // Step 2: whitelist or blacklist countries
            //         Countries are referenced by their IU code
            RoutingOption[] routingOption = new RoutingOption[] {
		        // Alternative 1: only allow routes through Germany and France
                //new RoutingOption { parameter = RoutingParameter.ROUTING_COUNTRIES, value = "49,33" }
                // Alternative 2: avoid routes through Luxemburg
		        new RoutingOption { parameter = RoutingParameter.EXCLUDE_COUNTRIES, value = "52" }
		    };

            //Step 2: Define start and destination
            xserver.Point startPoint = new xserver.Point();
            startPoint.point = new PlainPoint { x = 524730.47, y = 6397686.86 };

            WaypointDesc wpdStart = new WaypointDesc();
            wpdStart.wrappedCoords = new xserver.Point[] { startPoint };
            wpdStart.linkType = LinkType.NEXT_SEGMENT;
            xserver.Point destPoint = new xserver.Point();
            destPoint.point = new PlainPoint { x = 737927.84, y = 6397192.86 };

            WaypointDesc wpdDest = new WaypointDesc();
            wpdDest.wrappedCoords = new xserver.Point[] { destPoint };
            wpdDest.linkType = LinkType.NEXT_SEGMENT;

            WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };

            // Step 3: Define which results to get
            ResultListOptions resultListOptions = new ResultListOptions
            {
                polygon = true,
                totalRectangle = true,
                detailLevel = DetailLevel.STANDARD
            };

            try
            {
                // Step 4: calculate the route
                Route route = xRouteClient.calculateRoute(waypointDescs, routingOption, 
                        null, resultListOptions, null);

                // Step 5: generate the map, to display the route

                // Step 5.1: use bounding box of route
                BoundingBox boundingBox = new BoundingBox();
                boundingBox.leftTop = route.totalRectangle.leftBottom;
                boundingBox.rightBottom = route.totalRectangle.rightTop;

                // Step 5.2: get "polygon" from route
                LineString[] lineStrings = new LineString[] { route.polygon };

                // Step 5.3: set line options ( color, stroke size )
                LineOptions options = new LineOptions();

                LinePartOptions linePartOpt = new LinePartOptions {
                    color = new xserver.Color { red = 255, green = 0, blue = 0 },
                    visible = true,
                    width = -6
                };

                options.showFlags = true;

                Lines[] linesArray = new Lines[] { new Lines { wrappedLines = lineStrings, options = options} };

                // Step 5.4: put lines into a CustomLayer
                CustomLayer customLayer = new CustomLayer
                {
                    name = "routeLayer",
                    wrappedLines = linesArray,
                    drawPriority = 90, // this is PRIORROUTE
                    centerObjects = true,
                    objectInfos = ObjectInfoType.FULLGEOMETRYCLIPPED,
                    visible = true
                };

                // Step 5.5: : Create imageInfo and set relevant attributes
                ImageInfo imageInfo = new ImageInfo
                {
                    format = ImageFileFormat.GIF,
                    height = 600,
                    width = 800,
                    imageParameter = ""
                };
                
                MapParams mapParams = new MapParams { showScale = false, useMiles = false, language = "x-ptv-DFT" } ;

                //Step 6: Generate map
                Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams,
                        imageInfo, new Layer[] { customLayer }, false, null);

                // Step 7: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Step 8: Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // TODO proper exception handling
            }
        }
    }
}
