﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using RoutingWithItinerary.XRoute;

namespace RoutingWithItinerary
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        static String[] networkClassNames = new String[] {
			 "MOTORWAY", "HIGHWAY", "TRUNK_ROAD",
			 "COUNTRY_ROAD", "CITY_ROAD", "RESIDENTIAL_ROAD",
			 "SPECIAL_ROAD", "CYCLE_AND_WALKWAY" };
        
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: instantiate xRoute client. Use the packages utils.
            XRouteWSService xRouteClient = new XRouteWSService();
             // Step 2.1: Define way-points start and destination.
		    XRoute.Point startPoint = new XRoute.Point();
		    startPoint.point = new PlainPoint { x = 681593.765751686, y = 6371793.192390807 } ;
		    WaypointDesc wpdStart = new WaypointDesc();
		    wpdStart.wrappedCoords = new XRoute.Point[] { startPoint };
            wpdStart.linkType = LinkType.NEXT_SEGMENT;

		    XRoute.Point destPoint = new XRoute.Point();
            destPoint.point = new PlainPoint { x = 678909.5202224851, y = 6395906.712571196 } ;
		    WaypointDesc wpdDest = new WaypointDesc();
		    wpdDest.wrappedCoords = new XRoute.Point[] { destPoint };
		    wpdDest.linkType = LinkType.NEXT_SEGMENT;

		    // Step 2.2: Set start and destination into way-point description
		    WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };

		    // Step 3: Create resultListOptions. Set properties manouevres, texts,
		    // segments and detail level.
		    ResultListOptions resultListOptions = new ResultListOptions
            {
		        manoeuvres = true,
		        texts = true,
		        segments = true,
		        detailLevel = DetailLevel.STANDARD
            };

		    // Step 4: Set the routing option for language
		    RoutingOption[] routingOption = new RoutingOption[] { new RoutingOption { parameter = RoutingParameter.ROUTE_LANGUAGE, value = "EN" } };

            try
            {
                // Step 5: Return object route
                Route route = xRouteClient.calculateRoute(waypointDescs, routingOption, null, resultListOptions, null);
                RouteManoeuvre[] manoeuvers = route.wrappedManoeuvres;
                RouteListSegment[] _routeSegments = route.wrappedSegments;

                // Step 6.1: Create array of table entries
                RoutingTableEntry[] rows = new RoutingTableEntry[manoeuvers.Length];

                for (int i = 0; i < manoeuvers.Length; i++)
                {
                    // Step 6.2: Get street name, manoeuvre description, distance and time
                    int segmentId = manoeuvers[i].routeListSegmentIdx;
                    RouteListSegment segment = _routeSegments[segmentId];
                    int streetId = segment.streetNameIdx;
                    String stName = "";
                    if (streetId >= 0)
                        stName = route.wrappedTexts[streetId];
                    RoutingTableEntry row = new RoutingTableEntry
                    {
                        dist = segment.accDist / 1000.0,
                        time = segment.accTime / 60.0,
                        manDesc = manoeuvers[i].manoeuvreDesc,
                        streetName = stName
                    };
                    rows[i] = row;
                }

                // Step 7.1: Display all in table
                gridRoute.ItemsSource = rows;
            }
            catch (Exception e)
            {
                //TODO: proper exception handling
            }
        }
    }

    public class RoutingTableEntry
    {
        public double dist { get; set; }
        public double time { get; set; }
        public string manDesc { get; set; }
        public String streetName { get; set; }
    }
}
