﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingWithReachableObjects
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: Instantiate array of points, with points[0] = home point
            //         and some other points, that are checked fore reachability
            xserver.Point homePoint = new  xserver.Point();
		    homePoint.point = new PlainPoint { x = 681593.765751686, y = 6371793.192390807 } ;

            xserver.Point point1 = new xserver.Point();
            point1.point = new PlainPoint { x = 678909.0, y = 6395906.0 };

            xserver.Point point2 = new xserver.Point();
            point2.point = new PlainPoint { x = 678009.0, y = 6398906.0 };

            xserver.Point[] points = new xserver.Point[] { homePoint, point1, point2 };

            // Step 2: Calculate reach and draw the points on the map
            Reach reach = calculateReach(points);
            if (reach != null)
                drawReach(reach, points);
        }

        private Reach calculateReach(xserver.Point[] points) {
		    // Step 3: Instantiate xRoute client
            XRouteWSService xRouteClient = new XRouteWSService();
		
		    // Step 4: Define waypoints with previously generated points
		    // home waypoint
            WaypointDesc home = new WaypointDesc();
		    home.wrappedCoords =  new xserver.Point[]{points[0]};
		    home.linkType = LinkType.NEXT_SEGMENT;

            // waypoints, that are tested for reachability
		    WaypointDesc wpd1 = new WaypointDesc();
		    wpd1.wrappedCoords =  new xserver.Point[]{points[1]};
		    wpd1.linkType = LinkType.NEXT_SEGMENT;

		    WaypointDesc wpd2 = new WaypointDesc();
		    wpd2.wrappedCoords =  new xserver.Point[]{points[2]};
		    wpd2.linkType = LinkType.NEXT_SEGMENT;
		
		    WaypointDesc[] waypointDescs = new WaypointDesc[] {wpd1, wpd2};
		    //-------------------------------------------------
		
		    // Step 5: Define a distance/time constraint
		    ExpansionDescription expansionDescription =	new ExpansionDescription
            {
		        expansionType = ExpansionType.EXP_DIST,
		        wrappedHorizons = new int[] {20000}
            };
				
		    // => reachable are waypoints which can be reached from 'home' 
		    //    with a route shorter than 20km
		
		    try {
			     Reach reach = xRouteClient.calculateReachableObjects(
					    home, null, waypointDescs, null, expansionDescription, null);
                return reach;
		    }catch(Exception e){
			    // TODO: proper exception handling
		    }
            return null;
	    }

        private void drawReach(Reach reach, xserver.Point[] points)
        {
            
            // Step 6: Initialize xmap client
            XMapWSService xMapClient = new XMapWSService();

            // Step 7: Define mapSection and set center and scale
            MapSection mapSection = new MapSection();
            xserver.Point centerPoint = new xserver.Point { point = new PlainPoint { x = 685407.751736, y = 6372537.965244 } };
            mapSection.center = points[0];
            mapSection.scale = 1000;

            // Step 8: Create imageInfo and set relevant attributes
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 9: Realize object mapParams showing scale without use of miles.
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;

            // Step 10: Fill bitmapArray with start and destination flag
            Bitmap[] bitmapArray = new Bitmap[reach.wrappedReachInfo.Length + 1];

            bitmapArray[0] = new Bitmap
            {
                descr = "Reachable",
                position = points[0],
                name = "location.bmp"
            };
            int index = 1;
            foreach (ReachInfo ri in reach.wrappedReachInfo)
            {
                xserver.Point point = new xserver.Point();
                point.point = new PlainPoint { x = 0, y = 0 };
                if (ri.reachable == true)
                {
                    bitmapArray[index] = new Bitmap
                    {
                        descr = "Reachable",
                        position = points[index],
                        name = "flaggreen.bmp"
                    };
                }
                else
                {
                    bitmapArray[index] = new Bitmap
                    {
                        descr = "NotReachable",
                        position = points[index],
                        name = "flagred.bmp"
                    };
                }
                index++;
            }

            // Step 11: Set bitmapOptions
            BitmapOptions bitmapOptions = new BitmapOptions();
            // change the reference point for the pixel position of the bitmap
            // by default the bitmap center will be used.
            // here: bitmap displacement of 20 pixels.
            PlainPoint referencePoint = new PlainPoint { x = 20, y = 20 };
            bitmapOptions.referencePoint = referencePoint;
            bitmapOptions.transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 };

            // Step 12: Add bitmapArray and bitmapOptions into bitmaps
            Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps { wrappedBitmaps = bitmapArray, options = bitmapOptions } };

            // Step 13: Initialize customLayer with the relevant settings
            CustomLayer customLayer = new CustomLayer
            {
                centerObjects = true,
                drawPriority = 25000,
                name = "ObjectLayer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps
            };

            // Step 14: Put customLayer into layers
            Layer[] layers = new Layer[] { customLayer };

            // Step 15: Define boolean value including image in response
            bool includeImageInResponse = false;

            try
            {
                // Step 16: Return map object with its characteristics
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 17: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // proper exception handling: see API for PTV's exception handling!
            }
        }
    }
}
