﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingWithTimeCalc1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample() {
		    // Step 1: Instantiate xRoute client
            XRouteWSService xRouteClient = new XRouteWSService();
		
		    xserver.Point point;

		    //---------------------------------------------------------------------
		    // 1. Tour point: Hamburg
		
		    TourPointDesc hamburg = new TourPointDesc();
		    hamburg.linkType = LinkType.AUTO_LINKING;
		    point = new xserver.Point();
		    point.point = new PlainPoint { x = 1108401, y = 7078031};
		    hamburg.wrappedCoords = new xserver.Point[]{point };
		
		    //---------------------------------------------------------------------
		    // 2. Tour point: Leipzig
		
		    TourPointDesc leipzip = new TourPointDesc();
		    leipzip.linkType = LinkType.AUTO_LINKING;
		    point = new xserver.Point();
            point.point = new PlainPoint { x = 1378012, y = 6681209};
		    leipzip.wrappedCoords = new xserver.Point[]{point };
		
		    // it takes 1 hour to unload the truck in Leipzig
		    leipzip.servicePeriod = 3600;
		
		    // the driver does not have to help and can use the service period as break
		    leipzip.useServicePeriodForRecreation = true;
		
		    // the driver has to arrive within the opening hours
		    leipzip.wrappedOpeningIntervals = new Interval[]{
				    // from 2008-11-16 8:00h 
				    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 16,  8, 0, 0),
						    till = new System.DateTime(2008, 10, 16, 18, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    }
		    };
		
		     // the unloading does not need to be finished within the opening hours
		    leipzip.completeServiceInIntervals = false;
		    //---------------------------------------------------------------------
		
		    //---------------------------------------------------------------------
		    // 3. Tour point: Frankfurt
	
		    TourPointDesc frankfurt = new TourPointDesc();
		    frankfurt.linkType = LinkType.AUTO_LINKING;
		    point = new xserver.Point();
            point.point = new PlainPoint { x = 962659, y = 6461857};
		    frankfurt.wrappedCoords = new xserver.Point[]{point };
		    frankfurt.servicePeriod = 1800; // 30 minutes
		
		    frankfurt.wrappedOpeningIntervals = new Interval[]{
				    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 16,  8, 0, 0),
						    till = new System.DateTime(2008, 10, 16, 13, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    },
                    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 16, 15, 0, 0),
						    till = new System.DateTime(2008, 10, 16, 22, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    }
		    };
		    //---------------------------------------------------------------------
		
		
		    //---------------------------------------------------------------------
		    // 4. Tour point: Munich
	
		    TourPointDesc munich = new TourPointDesc();
		    munich.linkType = LinkType.AUTO_LINKING;
		    point = new xserver.Point();
            point.point = new PlainPoint { x = 1292573, y = 6125849};
		    munich.wrappedCoords = new xserver.Point[]{point };
		    munich.servicePeriod = 1800; // 30 minutes
		    munich.wrappedOpeningIntervals = new Interval[]{
				    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 17,  8, 0, 0),
						    till = new System.DateTime(2008, 10, 17, 13, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    },
                    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 17, 15, 0, 0),
						    till = new System.DateTime(2008, 10, 17, 20, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    }
		    };
		    //---------------------------------------------------------------------
		    // tourPoints is an array of stations defined by WaypointDesc or TourPointDesc objects
		    // first and last station MUST be of type TourPointDesc
		    WaypointDesc[] tourPoints = new WaypointDesc[]{ hamburg, leipzip, frankfurt, munich};
		
		    ResultListOptions resultListOptions = new ResultListOptions();
		    // set manoveurs to true, if itinerary with manoveurs is wanted
            // resultListOptions.setManoeuvres(true);
		    resultListOptions.texts = true;
		    resultListOptions.segments = true;
		    resultListOptions.detailLevel = DetailLevel.STANDARD;
            resultListOptions.polygon = true;
            resultListOptions.totalRectangle = true;


		    // ------ Setting START_TIME --------
		    // the point of time set in START_TIME will be used as reference for
		    // relative time specifications.
		    // If you do not set the START_TIME parameter, it will be set to 'now'
		    // disable RoadEditor

            System.DateTime earliestStartTime = new System.DateTime(2008, 10, 16, 8, 0, 0);
            RoutingOption routingOptionRE = new RoutingOption
            {
                parameter = RoutingParameter.ENABLE_ROADEDITOR,
                value = "false"
            };

            RoutingOption routingOptionStart = new RoutingOption
            {
                parameter = RoutingParameter.START_TIME,
                value = "2008-10-16T08:00:00"
            };
		
		    RoutingOption[] parameters = new RoutingOption[]{
                    routingOptionStart,
				    routingOptionRE
		    };
		
		    try {
			    Tour tour 
				    = xRouteClient.calculateTour(tourPoints, parameters, null, resultListOptions, null, null);
			
			    Route route = tour.route;
			
			    RouteListSegment[] routeSegments = route.wrappedSegments;
		
			    // the Texts array contains strings which are referenced
			    // by other data of the route (e.g. street names)
			    String[] texts = route.wrappedTexts;
		

			    // the TimeEvents array contains events, which (should) occur during the
			    // tour. Events include driving periods, rest periods, breaks, etc.
			    // The observable event types are documented in the API reference. 
			    TimeEvent[] events =  tour.wrappedTimeEvents;
			
                List<TourTableEntry> rows = new List<TourTableEntry>();
			
			    for (int i=0; i<events.Length; i++ )  {
				
				    String streetName = "";
				    // in order to get the street name we 
				    // check if the events references a valid segment ID
				    if (events[i].segmentIdx < routeSegments.Length) {
					    // get the reference segment
					    RouteListSegment segment = routeSegments[events[i].segmentIdx];
					    // get the street name ID from the segment
					    int streetId = segment.streetNameIdx;
					
					    // if street idx is valid, get the street name
					    // from the array of texts
					    if (streetId >= 0)
						    streetName = texts[streetId];
				    }

                    System.DateTime c = earliestStartTime;
				    // times are relative to START_TIME
                    System.TimeSpan duration = new System.TimeSpan(0, 0, events[i].startTime);
                    c = c.Add(duration);
                    System.DateTime startTime = c;

                    duration = new System.TimeSpan(0, 0, events[i].period);
                    c = c.Add(duration);
                    System.DateTime endTime = c;

                    int tourPoint = 0;
                    if (events[i].tourPointIdx >= 0)
                        tourPoint = events[i].tourPointIdx;
                    TourTableEntry row = new TourTableEntry
                    {
                        tourEvent = events[i].type,
                        tourPoint = tourPoint,
                        street = streetName,
                        start = startTime,
                        end = endTime
                    };

                    rows.Add(row);
			    }
			    // create a Table
                gridTour.ItemsSource = rows;
                displayMapOfRoute(route);
		
		    }catch(Exception e){
			    // TODO proper exception handling
		    }
	    }

        private void displayMapOfRoute(Route route)
        {
            // Step 1: Initialize client
            XMapWSService xMapClient = new XMapWSService();

            // Step 2: Define imageInfo
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 530;
            imageInfo.width = 787;
            imageInfo.imageParameter = "";

            // Step 3: Add mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;
            mapParams.language = "x-ptv-DFT"; //optional
            bool includeImageInResponse = false;

            // Step 4: Create a line from the given route
            LineString[] lineStrings = new LineString[] { route.polygon };
            Lines[] lines = new Lines[1];
            LineOptions options = new LineOptions();
            LinePartOptions linePartOpt = new LinePartOptions
            {
                color = new xserver.Color { red = 0, blue = 255, green = 0 },
                visible = true,
                width = -12
            };
            options.mainLine = linePartOpt;

            lines[0] = new Lines { wrappedLines = lineStrings, options = options };

            // Step 5: Define a custom layer that contains the line(s)
            CustomLayer customLayer = new CustomLayer
            {
                visible = true,
                drawPriority = 100, // this is PRIORROUTE
                wrappedLines = lines,
                objectInfos = ObjectInfoType.NONE,
                centerObjects = true
            };

            Layer[] layers = new Layer[] { customLayer};

            // Step 7: Set the bounding box
            BoundingBox boundingBox = new BoundingBox
            {
                leftTop = route.totalRectangle.leftBottom,
                rightBottom = route.totalRectangle.rightTop,
            };

            try
            {
                // Step 8: Call method renderMapBoundingBox
                Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse, null);
                // Step 10: retrieve and display the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
        }
    }

    public class TourTableEntry
    {
        public TimeEventType tourEvent { get; set; }
        public int tourPoint { get; set; }
        public String street { get; set; }
        public System.DateTime start { get; set; }
        public System.DateTime end { get; set; }
    }
}
