﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingWithTimeEvents
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample() {
            System.DateTime earliestStartTime = new System.DateTime(2008, 10, 16, 6, 0, 0);
		    // calculate a tour
		    // (the CalculateTour.java showed how this is done)
		    Tour tour = calculateSampleTour(earliestStartTime);
		
		    if (tour == null) {
			    return;
		    }

            // We use a Dictionary to map a list of TimeEvents to their route segment ID
		    // later, we use this to get the associated TimeEvents when we have a segment ID
		    Dictionary<int, List<TimeEvent>> eventsPerSegmentIdx = new Dictionary<int, List<TimeEvent>>();

            // go through all TimeEvents and add them to the Dictionary ...
		    foreach (TimeEvent tEvent in tour.wrappedTimeEvents) {
			    int key = tEvent.segmentIdx;
			    List<TimeEvent> eventList;
			
			    if ( eventsPerSegmentIdx.ContainsKey(key) ) {
				    eventList = eventsPerSegmentIdx[key];
			    }else{
				    eventList = new List<TimeEvent>();
				    eventsPerSegmentIdx[key] = eventList;
			    }
			    eventList.Add(tEvent);
		    }
		
		
		    // we want to display the data in a Table, 
		    // so we store the data in a Vector of rows
		    List<TourTableEntry> tableRows = new List<TourTableEntry>();
		
		    // loop through all manoeuvres ...
		    foreach (RouteManoeuvre manoeuvre in tour.route.wrappedManoeuvres) {
                int key = manoeuvre.routeListSegmentIdx;
			
			    // skip manoeuvres that have no associated TimeEvent 
			    // AND have a different DetailLevel than VERY_IMPORTANT
			    if ( !eventsPerSegmentIdx.ContainsKey(key) 
					    && manoeuvre.detailLevel != DetailLevel.VERY_IMPORTANT)
				    continue;
			
			    StringBuilder sb = new StringBuilder();
			    // if the segment of the current manoeuvre has an 
			    // associated list of time events we create a comma
			    // seperated list
			    if ( eventsPerSegmentIdx.ContainsKey(key) ) {
				    String pre = "";
				    foreach (TimeEvent e in eventsPerSegmentIdx[key]) {
					    sb.Append(pre + e.type.ToString());
					    pre = ", ";
				    }
			    }
                TourTableEntry row = new TourTableEntry 
                {
                    desc = manoeuvre.manoeuvreDesc,
                    manType = manoeuvre.manoeuvreType,
                    tourEvent = sb.ToString()
                };
			    tableRows.Add(row);
            }

            gridTour.ItemsSource = tableRows;
	    }
	
	
	    /**
	     * Calculates a sample tour 
         * 
	     * @param startTime the start time
	     * 
	     * @return the tour
	     */
        private Tour calculateSampleTour(System.DateTime startTime)
        {
		     // Step 1: Instantiate xRoute client
            XRouteWSService xRouteClient = new XRouteWSService();
		
		    xserver.Point point;

		    //---------------------------------------------------------------------
		    // 1. Tour point: Hamburg
		
		    TourPointDesc hamburg = new TourPointDesc();
		    hamburg.linkType = LinkType.AUTO_LINKING;
		    point = new xserver.Point();
		    point.point = new PlainPoint { x = 1108401, y = 7078031};
		    hamburg.wrappedCoords = new xserver.Point[]{point };
		
		    //---------------------------------------------------------------------
		    // 2. Tour point: Leipzig
		
		    TourPointDesc leipzip = new TourPointDesc();
		    leipzip.linkType = LinkType.AUTO_LINKING;
		    point = new xserver.Point();
            point.point = new PlainPoint { x = 1378012, y = 6681209};
		    leipzip.wrappedCoords = new xserver.Point[]{point };
		
		    // it takes 1 hour to unload the truck in Leipzig
		    leipzip.servicePeriod = 3600;
		
		    // the driver does not have to help and can use the service period as break
		    leipzip.useServicePeriodForRecreation = true;
		
		    // the driver has to arrive within the opening hours
		    leipzip.wrappedOpeningIntervals = new Interval[]{
				    // from 2008-11-16 8:00h 
				    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 16,  8, 0, 0),
						    till = new System.DateTime(2008, 10, 16, 18, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    }
		    };
		
		     // the unloading does not need to be finished within the opening hours
		    leipzip.completeServiceInIntervals = false;
		    //---------------------------------------------------------------------
		
		    //---------------------------------------------------------------------
		    // 3. Tour point: Frankfurt
	
		    TourPointDesc frankfurt = new TourPointDesc();
		    frankfurt.linkType = LinkType.AUTO_LINKING;
		    point = new xserver.Point();
            point.point = new PlainPoint { x = 962659, y = 6461857};
		    frankfurt.wrappedCoords = new xserver.Point[]{point };
		    frankfurt.servicePeriod = 1800; // 30 minutes
		
		    frankfurt.wrappedOpeningIntervals = new Interval[]{
				    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 16,  8, 0, 0),
						    till = new System.DateTime(2008, 10, 16, 13, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    },
                    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 16, 15, 0, 0),
						    till = new System.DateTime(2008, 10, 16, 22, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    }
		    };
		    //---------------------------------------------------------------------
		
		
		    //---------------------------------------------------------------------
		    // 4. Tour point: Munich
	
		    TourPointDesc munich = new TourPointDesc();
		    munich.linkType = LinkType.AUTO_LINKING;
		    point = new xserver.Point();
            point.point = new PlainPoint { x = 1292573, y = 6125849};
		    munich.wrappedCoords = new xserver.Point[]{point };
		    munich.servicePeriod = 1800; // 30 minutes
		    munich.wrappedOpeningIntervals = new Interval[]{
				    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 17,  8, 0, 0),
						    till = new System.DateTime(2008, 10, 17, 13, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    },
                    new Interval 
                    {
						    from =  new System.DateTime(2008, 10, 17, 15, 0, 0),
						    till = new System.DateTime(2008, 10, 17, 20, 0, 0),
                            fromSpecified = true,
                            tillSpecified = true
				    }
		    };
		    //---------------------------------------------------------------------
		    // tourPoints is an array of stations defined by WaypointDesc or TourPointDesc objects
		    // first and last station MUST be of type TourPointDesc
		    WaypointDesc[] tourPoints = new WaypointDesc[]{ hamburg, leipzip, frankfurt, munich};
		
		    ResultListOptions resultListOptions = new ResultListOptions();
		    // set manoveurs to true, if itinerary with manoveurs is wanted
            // resultListOptions.setManoeuvres(true);
		    resultListOptions.texts = true;
		    resultListOptions.segments = true;
		    resultListOptions.detailLevel = DetailLevel.STANDARD;
            resultListOptions.polygon = true;
            resultListOptions.totalRectangle = true;
            resultListOptions.manoeuvres = true;


		    // ------ Setting START_TIME --------
		    // the point of time set in START_TIME will be used as reference for
		    // relative time specifications.
		    // If you do not set the START_TIME parameter, it will be set to 'now'
		    // disable RoadEditor

            System.DateTime earliestStartTime = new System.DateTime(2008, 10, 16, 8, 0, 0);
            RoutingOption routingOptionRE = new RoutingOption
            {
                parameter = RoutingParameter.ENABLE_ROADEDITOR,
                value = "false"
            };

            RoutingOption routingOptionStart = new RoutingOption
            {
                parameter = RoutingParameter.START_TIME,
                value = String.Format("{0:s}", startTime)
            };
		
		    RoutingOption[] parameters = new RoutingOption[]{
                    routingOptionStart,
				    routingOptionRE
		    };
		
		    try {
			    Tour tour
                    = xRouteClient.calculateTour(tourPoints, parameters, null, resultListOptions, null, null);
			    return tour;
		    }catch(Exception e){
			    // TODO proper exception handling
			    return null;
		    }
	    }
    }

    public class TourTableEntry
    {
        public String desc { get; set; }
        public ManoeuvreType manType { get; set; }
        public String tourEvent { get; set; }
    }
}
