﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using xserver;

namespace Geometry
{

    /**
     * Represents an edge in a graph, given as two points
     * The ID of this edge is calculated, by using the cantor pairing function on the IDs of the points
    **/
    class Edge
    {
        public Point P { get; private set; }
        public Point Q { get; private set; }
        public bool chiFinished { get; private set; }
        public List<Triangle> triangles { get; private set; }
        public int ID { get; private set; }
        private double length { get; set; }

        public Edge(Point P, Point Q)
        {
            this.P = P;
            this.Q = Q;
            calculateID();
            triangles = new List<Triangle>();
            chiFinished = false;
            length = -1;
        }

        // Tests, if the edge is equal to another (i.e. the two points are equal)
        public bool isEqual(Edge e2)
        {
            PointTest pt = new PointTest();
            if ((pt.areEqual(e2.P, P) && pt.areEqual(e2.Q, Q)) || (pt.areEqual(e2.Q, P) && pt.areEqual(e2.P, Q)))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        // Checks, if two edges have the same ID
        public bool isEqualByID(Edge e2)
        {
            if (this.ID == e2.ID)
                return true;
            else
                return false;
        }

        // calculates the length of the edge
        public double getLength()
        {
            if (this.length == -1)
                this.length = Math.Sqrt(Math.Pow(P.pt.x - Q.pt.x, 2) + Math.Pow(P.pt.y - Q.pt.y, 2));
            return this.length;
        }

        /**
         * Calculates the ID of the edge, using the cantor pairing function with the IDs of the two points
         * The smaller point ID is used as x, the larger as y, to ensure the same edge cannot get different IDs
         * (http://en.wikipedia.org/wiki/Pairing_function)
        **/
        private void calculateID()
        {
            int x = Math.Min(P.ID, Q.ID);
            int y = Math.Max(P.ID, Q.ID);
            this.ID = Convert.ToInt32(y + 0.5 * (x + y) * (x + y + 1));
        }

        public void finish()
        {
            this.chiFinished = true;
        }
    }
}
