﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using xserver;

namespace Geometry
{
    /**
     * Represents a triangle, given as a set of 3 Points and a unique ID
    **/
    class Triangle
    {
        public Point A { get; set; }
        public Point B { get; set; }
        public Point C { get; set; }
        private Circle circumCircle { get; set; }
        public int ID { get; set; }
        public List<Edge> edges { get; set; }
        public bool bad { get; set; }

        public Triangle(Point A, Point B, Point C, int ID)
        {
            this.A = A;
            this.B = B;
            this.C = C;
            this.ID = ID;
            edges = new List<Edge>();

            calculateCircumCircle();

            edges.Add(new Edge(A, B));
            edges.Add(new Edge(A, C));
            edges.Add(new Edge(B, C));

            if (!circumCircle.isInside(A) || !circumCircle.isInside(B) || !circumCircle.isInside(C))
            {
                circumCircle.isInside(A);
                circumCircle.isInside(B);
                circumCircle.isInside(C);
            }

            bad = false;
        }

        // Calculates the circumcircle of the triangle
        private void calculateCircumCircle()
        {
            PlainPoint center = new PlainPoint();
            double D = 2 * (A.pt.x * (B.pt.y - C.pt.y) + B.pt.x * (C.pt.y - A.pt.y) + C.pt.x * (A.pt.y - B.pt.y));
            center.x = ((Math.Pow(A.pt.x, 2) + Math.Pow(A.pt.y, 2)) * (B.pt.y - C.pt.y) + (Math.Pow(B.pt.x, 2) + Math.Pow(B.pt.y, 2)) * (C.pt.y - A.pt.y) + (Math.Pow(C.pt.x, 2) + Math.Pow(C.pt.y, 2)) * (A.pt.y - B.pt.y)) / D;
            center.y = ((Math.Pow(A.pt.x, 2) + Math.Pow(A.pt.y, 2)) * (C.pt.x - B.pt.x) + (Math.Pow(B.pt.x, 2) + Math.Pow(B.pt.y, 2)) * (A.pt.x - C.pt.x) + (Math.Pow(C.pt.x, 2) + Math.Pow(C.pt.y, 2)) * (B.pt.x - A.pt.x)) / D;
            double radius = Math.Sqrt(Math.Pow(A.pt.x - center.x, 2) + Math.Pow(A.pt.y - center.y, 2));

            circumCircle = new Circle(center, radius);
        }

        // Checks if the triangle is bad (with resprect to delaunay triangulations), i.e. contains p in its circumcircle
        public bool isBad(Point p)
        {
            if (circumCircle.isInside(p))
                return true;
            else
                return false;
        }

        // Checks, if a triangle contains a certain edge
        public bool containsEdge(Edge e)
        {
            foreach (Edge e2 in edges)
            {
                if (e2.isEqual(e))
                    return true;
            }
            return false;
        }

        // Scales a triangle by a given factor, using the center as origin (http://www.bbc.co.uk/schools/gcsebitesize/maths/images/graph_68.gif)
        public void scale(double factor, PlainPoint center)
        {
            A.pt.x = ((A.pt.x - center.x) * factor) + center.x;
            A.pt.y = ((A.pt.y - center.y) * factor) + center.y;
            B.pt.x = ((B.pt.x - center.x) * factor) + center.x;
            B.pt.y = ((B.pt.y - center.y) * factor) + center.y;
            C.pt.x = ((C.pt.x - center.x) * factor) + center.x;
            C.pt.y = ((C.pt.y - center.y) * factor) + center.y;
        }

        // Checks, if the triangle shares a node with another triangle
        public bool containsNodeFrom(Triangle tri)
        {
            PointTest pt = new PointTest();
            if (pt.areEqual(A, tri.A) || pt.areEqual(A, tri.B) || pt.areEqual(A, tri.C) || pt.areEqual(B, tri.A) || pt.areEqual(B, tri.B) || pt.areEqual(B, tri.C) || pt.areEqual(C, tri.A) || pt.areEqual(C, tri.B) || pt.areEqual(C, tri.C))
                return true;
            else
                return false;
        }

        // Given an edge, returns the third point in the triangle, by using the IDs
        public Point getThirdPointByID(Edge e)
        {
            PointTest pointTest = new PointTest();
            if (e.isEqualByID(edges[0]))
            {
                return C;
            }
            else if (e.isEqualByID(edges[1]))
            {
                return B;
            }
            else if (e.isEqualByID(edges[2]))
            {
                return A;
            }
            else
            {
                return null;
            }
        }

    }
}
