﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using xserver;

namespace Geometry
{
    class GrahamScan
    {
        public List<Location> calculateConvexHull(List<Location> locations)
        {
            List<Location> hullLocations = new List<Location>();
            List<Location> sortedLocations = new List<Location>(locations);

            //Determine location with the minimal y-coordinate, if there are multiple ones take the leftmost
            double minY = Double.MaxValue;
            double minX = Double.MaxValue;
            int index = 0;
            int i = 0;
            Location minLocation;
            foreach (Location loc in locations)
            {
                if (loc.coordinate.point.y < minY)
                {
                    index = i;
                    minY = loc.coordinate.point.y;
                    minX = loc.coordinate.point.x;
                }
                else if ((loc.coordinate.point.y < minY) && loc.coordinate.point.x < minX)
                {
                    index = i;
                    minY = loc.coordinate.point.y;
                    minX = loc.coordinate.point.x;
                }
                i++;
            }
            minLocation = locations[index];

            //Sort locations ascending, by the angle they form with the minLocation and the x-axis
            SorterByAngle sorter = new SorterByAngle(minLocation);
            sortedLocations.Sort(sorter.CompareLocations);

            //Add first two locations to the hull
            hullLocations.Add(sortedLocations[0]);
            hullLocations.Add(sortedLocations[1]);

            int j = 2;
            while (j < sortedLocations.Count)
            {
                //If the current location and the previous two form a counter-clockwise turn, add the location to the hull, increment the counter
                if (ccw(hullLocations[hullLocations.Count - 2], hullLocations[hullLocations.Count - 1], sortedLocations[j]) > 0)
                {
                    hullLocations.Add(sortedLocations[j]);
                    j++;
                }
                //If they are colinear, delete the previous location and add this one to the hull, increment the counter
                else if (ccw(hullLocations[hullLocations.Count - 2], hullLocations[hullLocations.Count - 1], sortedLocations[j]) == 0)
                {
                    hullLocations.RemoveAt(hullLocations.Count - 1);
                    hullLocations.Add(sortedLocations[j]);
                    j++;
                }
                //If the three locations form a clockwise turn, delete the previous location from the hull, do not increment the counter, repeat with the now update previous two locations
                else
                {
                    hullLocations.RemoveAt(hullLocations.Count - 1);
                }
            }

            return hullLocations;
        }

        //Checks, if three locations form a counter-clockwise turn, return-value > 0 -> counter-clockwise; return-value = 0 -> colinear; return-value < 0 -> clockwise
        private double ccw(Location l1, Location l2, Location l3)
        {
            return (l2.coordinate.point.x - l1.coordinate.point.x) * (l3.coordinate.point.y - l1.coordinate.point.y) - (l2.coordinate.point.y - l1.coordinate.point.y) * (l3.coordinate.point.x - l1.coordinate.point.x);
        }
    }
}
