﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace TourPlanningBasic
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public static double[,] tourPointsXY = new double[11,3];

        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            Plan plan = runTourSample();
            if (plan != null)
            {
                BitmapImage image = new BitmapImage();
                List<List<TourTableEntry>> tourOutput = new List<List<TourTableEntry>>();
                runTourOutput(plan, ref tourOutput);
                runMappingSample(plan, ref image, ref tourOutput);
                img1.Source = image;

                gridTourOutput.ItemsSource = tourOutput[2];
                gridTour1.ItemsSource = tourOutput[0];
                gridTour2.ItemsSource = tourOutput[1];
            }
        }

        private BitmapImage runMappingSample(Plan plan, ref BitmapImage image, ref List<List<TourTableEntry>> tourOutput) {
		    // Step 1: Instantiate xMap client.
            XMapWSService xMapClient = new XMapWSService();

		    // Step 2: Use bounding box to draw the calculated route.
		    BoundingBox boundingBox = new BoundingBox();
		    xserver.Point leftBottom = new xserver.Point { point = new PlainPoint { x = 653000, y = 6356000 } };
		    boundingBox.leftTop = leftBottom;
            xserver.Point rightTop = new xserver.Point { point = new PlainPoint { x = 697000, y = 6391000 } };
		    boundingBox.rightBottom = rightTop;

            int maxTourPoints = tourPointsXY.GetLength(0);
		    Bitmap[] bitmapTourPoints = new Bitmap[maxTourPoints];

		    for (int i = 0; i < maxTourPoints; i++) {
			    xserver.Point point = new xserver.Point();
			    point.point = new PlainPoint { x = tourPointsXY[i,0], y = tourPointsXY[i,1] };

			    if (i == 0) {
				    Bitmap flaggreen = new Bitmap
                    {
                        descr = "depot flag",
                        name = "flaggreen.bmp",
                        position = point
                    };
				    bitmapTourPoints[i] = flaggreen;
			    } else if (0 < i && i < 5) {
				    Bitmap flagblue = new Bitmap
                    {
                        descr = "transportDepot flag",
                        name = "flagblue.bmp",
                        position = point
                    };
				    bitmapTourPoints[i] = flagblue;
			    } else {
				    Bitmap flagred = new Bitmap
                    {
                        descr = "transportAB flag",
                        name = "flagred.bmp",
                        position = point
                    };
				    bitmapTourPoints[i] = flagred;
			    }
		    }

		    // Step 3.1: Define bitmap options
		    BitmapOptions bitmapOptions = new BitmapOptions();
		    bitmapOptions.transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 } ;

		    // Step 2.3: Define array and put bitmapReachableObjects and
		    // bitmapOptions into
		    // bitmaps
		    Bitmaps[] bitmaps = new Bitmaps[] { new Bitmaps { wrappedBitmaps = bitmapTourPoints, options = bitmapOptions } };

		    // Step 3: Define custom layer for bitmaps
		    CustomLayer customLayerBitmaps = new CustomLayer
            {
                name = "Draw bitmaps",
                wrappedBitmaps = bitmaps,
                drawPriority = 10000,
                centerObjects = true,
                objectInfos = ObjectInfoType.NONE,
                visible = true
            };
		    // Step 3: define an array of lines
		    int maxTour1 = tourOutput[0].Count;
		    int maxTour2 = tourOutput[1].Count;
		    Lines[] lines1 = new Lines[maxTour1];
		    Lines[] lines2 = new Lines[maxTour2];
		    List<TourTableEntry> rows1 = tourOutput[0];
		    List<TourTableEntry> rows2 = tourOutput[1];
		
		    for (int i = 0; i < maxTour1-1; i++) {
			    // Step 3.1: create plainLineString
			    PlainLineString plainLineString = new PlainLineString();
			    PlainPoint plainPointStart = new PlainPoint();
			    PlainPoint plainPointDestination = new PlainPoint();
			    int loc1 = rows1[i].orderID;
			    int loc2 = rows1[i+1].orderID;
			    double[,] xy1 = getLocation(loc1);
			    plainPointStart.x = xy1[0,0];
			    plainPointStart.y = xy1[0,1];
			    double[,] xy2 = getLocation(loc2);
			    plainPointDestination.x = xy2[0,0];
			    plainPointDestination.y = xy2[0,1];
			    plainLineString.wrappedPoints = new PlainPoint[] {plainPointStart, plainPointDestination};
			
			    // Step 3.2: set object into lineStrings[0]. Note: object lineStrings has
			    // to be reinitialized in every loop.
			    LineString[] lineStrings = new LineString[1];
			    lineStrings[0] = new LineString();
			    lineStrings[0].lineString = plainLineString;
			
			    // Step 3.3: create new lines[i] and fill array
			    lines1[i] = new Lines();
			    lines1[i].wrappedLines = lineStrings;
			
			    // Step 3.4: set lineOptions transparency
			    LineOptions lineOptions = new LineOptions();
			    lineOptions.transparent = true;
			
			    // Step 3.5: set linePartOptions visible, define width of line and
			    // specific color
			    LinePartOptions linePartOptions = new LinePartOptions();
			    linePartOptions.visible = true;
			    linePartOptions.width = 5;
			    linePartOptions.color = new xserver.Color { red = 255, green = 0, blue = 0 };
			
			    // Step 3.6: set main line
			    lineOptions.mainLine = linePartOptions;
			
			    // Step 3.7: set all options for specific line
			    lines1[i].options = lineOptions;	
		    }
		
		    for (int i = 0; i < maxTour2-1; i++) {
			    // Step 3.1: create plainLineString
			    PlainLineString plainLineString = new PlainLineString();
			    PlainPoint plainPointStart = new PlainPoint();
			    PlainPoint plainPointDestination = new PlainPoint();
			    int loc1 = rows2[i].orderID;
			    int loc2 = rows2[i+1].orderID;
			    double[,] xy1 = getLocation(loc1);
			    plainPointStart.x = xy1[0,0];
			    plainPointStart.y = xy1[0,1];
                double[,] xy2 = getLocation(loc2);
			    plainPointDestination.x = xy2[0,0];
			    plainPointDestination.y = xy2[0,1];
			    plainLineString.wrappedPoints = new PlainPoint[] {plainPointStart, plainPointDestination};
			
			    // Step 3.2: set object into lineStrings[0]. Note: object lineStrings has
			    // to be reinitialized in every loop.
			    LineString[] lineStrings = new LineString[1];
			    lineStrings[0] = new LineString();
			    lineStrings[0].lineString = plainLineString;
			
			    // Step 3.3: create new lines[i] and fill array
			    lines2[i] = new Lines();
			    lines2[i].wrappedLines = lineStrings;
			
			    // Step 3.4: set lineOptions transparency
			    LineOptions lineOptions = new LineOptions();
			    lineOptions.transparent = true;
			
			    // Step 3.5: set linePartOptions visible, define width of line and
			    // specific color
			    LinePartOptions linePartOptions = new LinePartOptions();
			    linePartOptions.visible = true;
			    linePartOptions.width = 5;
			    linePartOptions.color = new xserver.Color { red = 0, green = 0, blue = 255 } ;
			
			    // Step 3.6: set main line
			    lineOptions.mainLine = linePartOptions;
			
			    // Step 3.7: set all options for specific line
			    lines2[i].options = lineOptions;	
		    }

            CustomLayer customLayerTourOutput1 = new CustomLayer
            {
                name = "Draw lines1",
                wrappedLines = lines1,
                drawPriority = 500,
                centerObjects = true,
                objectInfos = ObjectInfoType.NONE,
                visible = true
            };

            CustomLayer customLayerTourOutput2 = new CustomLayer
            {
                name = "Draw lines2",
                wrappedLines = lines2,
                drawPriority = 500,
                centerObjects = true,
                objectInfos = ObjectInfoType.NONE,
                visible = true
            };

		    // Step 4: Define array and put customLayer into layers
		    Layer[] layers = new Layer[] { customLayerBitmaps, customLayerTourOutput1, customLayerTourOutput2 };

		    // Step 5: Create class imageInfo and set format and dimension.
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 530,
                width = 787
            };

		    // Step 6: Use mapParams for scale and distance unit
            MapParams mapParams = new MapParams
            {
                showScale = true,
                useMiles = false
            };

            try
            {
                // Step 7: Call renderMapBoundingBox and return object map.
                Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, false, null);

                // Step 10: retrieve and display the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                image = new BitmapImage();

                // Display BitmapImage
                image.BeginInit();
                image.UriSource = new Uri(imageUrl);
                image.EndInit();

                return image;

            }
            catch (Exception e)
            {
                //TODO: proper exception handling
                return null;
            }
	    }

        private double[,] getLocation(int tourPoint){
            double[,] xy = new double[1, 2];
		
		    for (int i = 0; i < tourPointsXY.GetLength(0); i++) {
			    if (tourPoint == tourPointsXY[i,2]){
				    xy[0,0] = tourPointsXY[i,0];
				    xy[0,1] = tourPointsXY[i,1];
			    }
		    }
		
		    return xy;
	    }

        private List<List<TourTableEntry>> runTourOutput(Plan plan, ref List<List<TourTableEntry>> TourTables) {
            List<TourTableEntry> rowsA = new List<TourTableEntry>();
		    foreach (Chain chain in plan.wrappedChains) {
                TourTableEntry rowA = new TourTableEntry
                {
                    vehicleID = chain.vehicleId,
                    drivingPeriod = second2hours(chain.result.drivingPeriod)
                };
			    
                foreach (Tour tour in chain.wrappedTours) {
				    List<TourTableEntry> rowsB = new List<TourTableEntry>();
				    foreach (TourPoint tourPoint in tour.wrappedTourPoints) {
                        TourTableEntry rowB = new TourTableEntry
                        {
                            vehicleID = chain.vehicleId,
                            tourID = tour.id,
                            orderID = tourPoint.id,
                            desc = tourPoint.description,
                            arrival = second2hours(tourPoint.result.arrivalTime),
                            departure = second2hours(tourPoint.result.departureTime),
                            service = second2hours(tourPoint.result.servicePeriod),
                            dist = tourPoint.result.drivingDistance / 1000.0,
                            drivingPeriod = second2hours(tourPoint.result.drivingPeriod),
                            quantitiy = tourPoint.result.currentQuantities.wrappedQuantities[0]
                        };
                        rowsB.Add(rowB);
				    }
                    TourTables.Add(rowsB);
			    }
                rowsA.Add(rowA);
		    }

            TourTables.Add(rowsA);

		    return TourTables;
	    }

        private String second2hours(int second)
        {
            String time = "";
            String hours = "";
            String minutes = "";
            String seconds = "";

            int h = (second / 3600);
            if (h < 10)
            {
                hours = "0" + h;
            }
            else
            {
                hours = "" + h;
            }

            int rest1 = (second % 3600);
            int min = (rest1 / 60);
            if (min < 10)
            {
                minutes = "0" + min;
            }
            else
            {
                minutes = "" + min;
            }

            int sec = (rest1 % 60);
            if (sec < 10)
            {
                seconds = "0" + sec;
            }
            else
            {
                seconds = "" + sec;
            }

            time = (hours + ":" + minutes + ":" + seconds);
            return time;
        }

        private Plan runTourSample() 
        {

		    // Step 1.1: instantiate xTour client
		    XTourWSService xTourClient = new XTourWSService();
		    // Step 1.2: Set a new CallerContext
		    CallerContext cxt = new CallerContext();
		    CallerContextProperty ccp1 = new CallerContextProperty
            {
                key = "ResponseGeometry",
                value = "WKT",
            };
            CallerContextProperty ccp2 = new CallerContextProperty
            {
                key = "Profile",
                value = "truck",
            };

		    CallerContextProperty[] properties = { ccp1, ccp2 };
		    cxt.wrappedProperties = properties;

		    // Step 2.1: Create object depot
		    Depot depotLux = new Depot();
		    depotLux.description = "Depot Luxembourg";
		    int depotID = 9999;
		    depotLux.id = depotID;

		    // Step 2.2: Set location
		    PlainPoint plainPointDepot = new PlainPoint();
		    tourPointsXY[0,0] = 681812.0;
		    tourPointsXY[0,1] = 6375071.0;
		    tourPointsXY[0,2] = depotID;
		    plainPointDepot.x = tourPointsXY[0,0];
		    plainPointDepot.y = tourPointsXY[0,1];
		    xserver.Point location = new xserver.Point();
		    location.point = plainPointDepot;
		    depotLux.location = location; // Rue Munchen-Tesch, Luxembourg

		    // Step 2.3: Define openingInterval for depot
		    Interval openingInterval = new Interval
            {
                // from 8:00 until 18:00
                from = 28800,
                till = 64800
            };
		    Interval[] openingIntervals = { openingInterval };
		    depotLux.wrappedOpeningIntervals = openingIntervals;

		    // Step 2.4: Put depotLux into the array depots
		    Depot[] depots = { depotLux };

		    // Step 3.1: Define transport depot 100 and set transport point 301
		    TransportDepot transportDepot100 = new TransportDepot 
            {
                id = 100,
                wrappedDepotIds = new int[] {depotID},
                //vehicleId = 1,
                //vehicleIdSpecified = true,
                wrappedVehicleCategories = new int[] { 1, 2, 3, 4, 5 },
                wrappedVehicleRequirements = new int[] { 1, 2, 3, 4, 5 },
                priority = 1,
                prioritySpecified = true,
            };

		    // Step 3.2: Define transportPoint 301
		    PlainPoint plainPoint301 = new PlainPoint();
		    tourPointsXY[1,0] = 682000.0;
		    tourPointsXY[1,1] = 6382000.0;
		    tourPointsXY[1,2] = 301;
		    plainPoint301.x = tourPointsXY[1,0];
		    plainPoint301.y = tourPointsXY[1,1];
		    xserver.Point point301 = new xserver.Point();
		    point301.point = plainPoint301;
		    TransportPoint transportPoint301 = new TransportPoint 
            {
                location = point301, // Rue Baron de Reinach, Heisdorf
                id =301,
                servicePeriod = 5800, 
                description = "shipment order 301: transport goods to customer 1",
               
            };

		    transportDepot100.transportPoint = transportPoint301;

		    // Step 3.3: Define quantities for delivery of shipment 301
		    Quantities deliveryQuantities301 = new Quantities();
		    int[] quantities301 = new int[] { 10, 0, 0, 0, 0, 0 };
		    deliveryQuantities301.wrappedQuantities = quantities301;
		    transportDepot100.deliveryQuantities = deliveryQuantities301;

		    // Step 4.1: Define transport depot 200 and set transport point 401
		    TransportDepot transportDepot200 = new TransportDepot
            {
                id = 200,
                wrappedDepotIds = new int[] { depotID },
                //vehicleId = 1,
                //vehicleIdSpecified = true,
                wrappedVehicleCategories = new int[] { 1, 2, 3, 4, 5 },
                wrappedVehicleRequirements = new int[] { 1, 2, 3, 4, 5 },
                priority = 1,
                prioritySpecified = true
            };

		    // Step 4.2: Define transportPoint 401
		    PlainPoint plainPoint401 = new PlainPoint();
		    tourPointsXY[2,0] = 671006.0;
		    tourPointsXY[2,1] = 6382294.0;
		    tourPointsXY[2,2] = 401;
		    plainPoint401.x = tourPointsXY[2,0];
		    plainPoint401.y = tourPointsXY[2,1];
		    xserver.Point point401 = new xserver.Point();
		    point401.point = plainPoint401;
		    TransportPoint transportPoint401 = new TransportPoint
            {
                location = point401, // Rue de Nospelt, Kehlen
                id = 401,
                servicePeriod = 5900,
                description = "shipment order 401: transport goods to customer 2",
            };
		    transportDepot200.transportPoint = transportPoint401;

		    // Step 4.3: Define quantities for delivery of shipment 401
		    Quantities deliveryQuantities401 = new Quantities();
		    int[] quantities401 = new int[] { 10, 0, 0, 0, 0, 0 };
		    deliveryQuantities401.wrappedQuantities = quantities401;
		    transportDepot200.deliveryQuantities = deliveryQuantities401;

		    // Step 5.1: Define transport depot 300 and set transport point 501
		    TransportDepot transportDepot300 = new TransportDepot
            {
                id = 300,
                wrappedDepotIds = new int[] { depotID },
                //vehicleId = 1,
                //vehicleIdSpecified = true,
                wrappedVehicleCategories = new int[] { 1, 2, 3, 4, 5 },
                wrappedVehicleRequirements = new int[] { 1, 2, 3, 4, 5 },
                priority = 1,
                prioritySpecified = true
            };

		    // Step 5.2: Define transportPoint 501
		    PlainPoint plainPoint501 = new PlainPoint();
		    tourPointsXY[3,0] = 682812.0;
		    tourPointsXY[3,1] = 6388220.0;
		    tourPointsXY[3,2] = 501;
		    plainPoint501.x = tourPointsXY[3,0];
		    plainPoint501.y = tourPointsXY[3,1];
		    xserver.Point point501 = new  xserver.Point();
		    point501.point = plainPoint501;
		    TransportPoint transportPoint501 = new TransportPoint();
		    transportPoint501.location = point501; // Rue Eugene Nickels,
		    // Lorentzweiler
		    transportPoint501.id = 501;
		    transportPoint501.servicePeriod = 6000;
		    transportPoint501.description = "shipment order 501: transport goods to customer 3";
		    transportDepot300.transportPoint = transportPoint501;

		    // Step 5.3: Define quantities for delivery of shipment 501
		    Quantities deliveryQuantities501 = new Quantities();
		    int[] quantities501 = new int[] { 10, 0, 0, 0, 0, 0 };
		    deliveryQuantities501.wrappedQuantities = quantities501;
		    transportDepot300.deliveryQuantities = deliveryQuantities501;

		    // Step 6.1: Define transport depot 400 and set transport point 601
		    TransportDepot transportDepot400 = new TransportDepot();
		    transportDepot400.id = 400;
		    transportDepot400.wrappedDepotIds = new int[] { depotID };
		    //transportDepot400.setVehicleId(1);
		    transportDepot400.wrappedVehicleCategories = new int[] { 1, 2, 3, 4, 5 };
		    transportDepot400.wrappedVehicleRequirements = new int[] { 1, 2, 3, 4, 5 };
		    transportDepot400.priority = 1;

		    // Step 6.2: Define transportPoint 601
		    PlainPoint plainPoint601 = new PlainPoint();
		    tourPointsXY[4,0] = 678665.0;
		    tourPointsXY[4,1] = 6356256.0;
		    tourPointsXY[4,2] = 601;
		    plainPoint601.x = tourPointsXY[4,0];
		    plainPoint601.y = tourPointsXY[4,1];
		    xserver.Point point601 = new xserver.Point();
		    point601.point = plainPoint601;
		    TransportPoint transportPoint601 = new TransportPoint();
		    transportPoint601.location = point601; // Rue des Cheminots,
		    // Bettembourg
		    transportPoint601.id = 601;
		    transportPoint601.servicePeriod = 6200;
		    transportPoint601.description = "shipment order 601: transport goods to customer 4";
		    transportDepot400.transportPoint = transportPoint601;

		    // Step 6.3: Define quantities for delivery of shipment 501
		    Quantities deliveryQuantities601 = new Quantities();
		    int[] quantities601 = new int[] { 10, 0, 0, 0, 0, 0 };
		    deliveryQuantities601.wrappedQuantities = quantities601;
		    transportDepot400.deliveryQuantities = deliveryQuantities601;

		    // Step 7.1: Define shipment AB 101
		    TransportAB transportAB101 = new TransportAB();
		    transportAB101.id = 101;
		    //transportAB101.setVehicleId(2);
		    transportAB101.wrappedVehicleCategories = new int[] { 1, 2, 3, 4, 5 };
		    transportAB101.wrappedVehicleRequirements = new int[] { 1, 2, 3, 4, 5 };
		    transportAB101.priority = 1;

		    // Step 7.2: Define transportPoint for location A1001 (pickup) and B1001
		    // (delivery).
		    PlainPoint plainPointA1001 = new PlainPoint();
		    tourPointsXY[5,0] = 681000.0;
		    tourPointsXY[5,1] = 6391000.0;
		    tourPointsXY[5,2] = 1001;
		    plainPointA1001.x = tourPointsXY[5,0];
		    plainPointA1001.y = tourPointsXY[5,1];
		    xserver.Point pointA1001 = new xserver.Point();
		    pointA1001.point = plainPointA1001;
		    TransportPoint transportPointA1001 = new TransportPoint();
		    transportPointA1001.id = 1001;
		    transportPointA1001.location = pointA1001; // Rue de Diekirch, Lintgen
		    transportPointA1001.servicePeriod = 3000;
		    transportPointA1001.description = "Pickup goods from location 1001";
		    transportAB101.transportPointA = transportPointA1001;

		    PlainPoint plainPointB1002 = new PlainPoint();
		    tourPointsXY[6,0] = 675566.0;
		    tourPointsXY[6,1] = 6380559.0;
		    tourPointsXY[6,2] = 1002;
		    plainPointB1002.x = tourPointsXY[6,0];
		    plainPointB1002.y = tourPointsXY[6,1];
		    xserver.Point pointB1002 = new xserver.Point();
		    pointB1002.point = plainPointB1002;
		    TransportPoint transportPointB1002 = new TransportPoint();
		    transportPointB1002.id = 1002;
		    transportPointB1002.location = pointB1002; // Rue des Bouleaux, Kopstal
		    transportPointB1002.servicePeriod = 3100;
		    transportPointB1002.description = "Deliver goods at location 1002";
		    transportAB101.transportPointB = transportPointB1002;

		    // Step 7.3: Define quantities for pickup and delivery A1001/B1002
		    Quantities quantitiesA1001B1002 = new Quantities();
		    int[] quantitiesAB100 = new int[] { 10, 0, 0, 0, 0, 0 };
		    quantitiesA1001B1002.wrappedQuantities = quantitiesAB100;
		    transportAB101.quantities = quantitiesA1001B1002;

		    // Step 8.1: Define shipment AB 201
		    TransportAB transportAB201 = new TransportAB();
		    transportAB201.id = 201;
		    //transportAB201.setVehicleId(2);
		    transportAB201.wrappedVehicleCategories = new int[] { 1, 2, 3, 4, 5 };
		    transportAB201.wrappedVehicleRequirements = new int[] { 1, 2, 3, 4, 5 };
		    transportAB201.priority = 1;

		    // Step 8.2: Define transportPoint for location A2001 (pickup) and B2001
		    // (delivery).
		    PlainPoint plainPointA2001 = new PlainPoint();
		    tourPointsXY[7,0] = 668207.0;
		    tourPointsXY[7,1] = 6390449.0;
		    tourPointsXY[7,2] = 2001;
		    plainPointA2001.x = tourPointsXY[7,0];
		    plainPointA2001.y = tourPointsXY[7,1];
		    xserver.Point pointA2001 = new xserver.Point();
		    pointA2001.point = plainPointA2001;
		    TransportPoint transportPointA2001 = new TransportPoint();
		    transportPointA2001.id = 2001;
		    transportPointA2001.location = pointA2001; // Rue de Brouch, Tuntange
		    transportPointA2001.servicePeriod = 3250;
		    transportPointA2001.description = "Pickup goods from location 2001";
		    transportAB201.transportPointA = transportPointA2001;

		    PlainPoint plainPointB2002 = new PlainPoint();
		    tourPointsXY[8,0] = 679108.0;
		    tourPointsXY[8,1] = 6396522.0;
		    tourPointsXY[8,2] = 2002;
		    plainPointB2002.x = tourPointsXY[8,0];
		    plainPointB2002.y = tourPointsXY[8,1];
		    xserver.Point pointB2002 = new xserver.Point();
		    pointB2002.point = plainPointB2002;
		    TransportPoint transportPointB2002 = new TransportPoint();
		    transportPointB2002.id = 2002;
		    transportPointB2002.location = pointB2002; // Rue Bouvart,
		    // Mersch
		    transportPointB2002.servicePeriod = 3300;
		    transportPointB2002.description = "Deliver goods at location 2002";
		    transportAB201.transportPointB = transportPointB2002;

		    // Step 8.3: Define quantities for pickup and delivery A2001/B2002
		    Quantities quantitiesA2001B2002 = new Quantities();
		    int[] quantitiesAB200 = new int[] { 10, 0, 0, 0, 0, 0 };
		    quantitiesA2001B2002.wrappedQuantities = quantitiesAB200;
		    transportAB201.quantities = quantitiesA2001B2002;

		    // Step 9.1: Define shipment AB 301
		    TransportAB transportAB301 = new TransportAB();
		    transportAB301.id = 301;
		    //transportAB301.setVehicleId(2);
		    transportAB301.wrappedVehicleCategories = new int[] { 1, 2, 3, 4, 5 };
		    transportAB301.wrappedVehicleRequirements = new int[] { 1, 2, 3, 4, 5 };
		    transportAB301.priority = 1;

		    // Step 9.2: Define transportPoint for location A3001 (pickup) and B3001
		    // (delivery).
		    PlainPoint plainPointA3001 = new PlainPoint();
		    tourPointsXY[9,0] = 653966.0;
		    tourPointsXY[9,1] = 6369424.0;
		    tourPointsXY[9,2] = 3001;
		    plainPointA3001.x = tourPointsXY[9,0];
		    plainPointA3001.y = tourPointsXY[9,1];
		    xserver.Point pointA3001 = new xserver.Point();
		    pointA3001.point = plainPointA3001;
		    TransportPoint transportPointA3001 = new TransportPoint();
		    transportPointA3001.id = 3001;
		    transportPointA3001.location = pointA3001; // Rue de Fingig, Clemency
		    transportPointA3001.servicePeriod = 3400;
		    transportPointA3001.description = "Pickup goods from location 3001";
		    transportAB301.transportPointA = transportPointA3001;

		    PlainPoint plainPointB3002 = new PlainPoint();
		    tourPointsXY[10,0] = 696741.0;
		    tourPointsXY[10,1] = 6373860.0;
		    tourPointsXY[10,2] = 3002;
		    plainPointB3002.x = tourPointsXY[10,0];
		    plainPointB3002.y = tourPointsXY[10,1];
		    xserver.Point pointB3002 = new xserver.Point();
		    pointB3002.point = plainPointB3002;
		    TransportPoint transportPointB3002 = new TransportPoint();
		    transportPointB3002.id = 3002;
		    transportPointB3002.location = pointB3002; // Rue de Neuhausen,
		    // Schuttrange
		    transportPointB3002.servicePeriod = 3600;
		    transportPointB3002.description = "Deliver goods at location 3002";
		    transportAB301.transportPointB = transportPointB3002;

		    // Step 9.3: Define quantities for pickup and delivery A3001/B3002
		    Quantities quantitiesA3001B3002 = new Quantities();
		    int[] quantitiesAB300 = new int[] { 10, 0, 0, 0, 0, 0 };
		    quantitiesA3001B3002.wrappedQuantities = quantitiesAB300;
		    transportAB301.quantities = quantitiesA3001B3002;

		    // Step 9.4: Create object transportOrders and fill with all
		    // transportDepot
		    // and transportAB objects.
		    // This is required for planBasicTours().
		    TransportOrder[] transportOrders = { transportDepot100, transportDepot200, transportDepot300, transportDepot400,
				    transportAB101, transportAB201, transportAB301 };

		    // Step 10.1: Create two new instances for the specified vehicle types
		    // which should be used.
		    Fleet fleet = new Fleet();
		    // Step 10.2: Define properties for vehicle one.
		    Vehicle vehicle1 = new Vehicle();
		    vehicle1.id = 1;
		    vehicle1.depotIdStart = depotID;
		    vehicle1.depotIdEnd = depotID;
		    vehicle1.isPreloaded = false;
		    vehicle1.description = "Truck Type A";
		    vehicle1.category = 1;
		    VehicleRestrictions vehRestrictions1 = new VehicleRestrictions();
		    vehRestrictions1.maxTourCountPerChain = 1;
		    vehRestrictions1.maxDrivingPeriodPerChain = 28800;
		    vehRestrictions1.maxTransportPointCountPerTour = 10;
		    vehicle1.restrictions = vehRestrictions1;
		    vehicle1.wrappedTransportEquipments = new int[] { 1, 2, 3, 4, 5 };
		    Capacities capacities1 = new Capacities();
		    Quantities quantity1 = new Quantities();
		    quantity1.wrappedQuantities = new int[] { 80, 0, 0, 0, 0, 0, 0 };
		    Quantities[] quantities1 = new Quantities[] { quantity1 };
		    capacities1.wrappedCapacities = quantities1;
		    vehicle1.capacities = capacities1;
		    VehicleCosts costs1 = new VehicleCosts();
		    costs1.costsPerHourDriverOnly = 80.0;
		    costs1.costsPerKm = 20.0;
		    vehicle1.costs = costs1;
		    vehicle1.dimaId = 1000;
            vehicle1.dimaIdSpecified = true;

		    // Step 10.3: Define properties for vehicle two.
		    Vehicle vehicle2 = new Vehicle();
		    vehicle2.id = 2;
		    vehicle2.depotIdStart = depotID;
		    vehicle2.depotIdEnd = depotID;
		    vehicle2.isPreloaded = false;
		    vehicle2.description = "Truck Type B";
		    vehicle2.category = 2;
		    VehicleRestrictions vehRestrictions2 = new VehicleRestrictions();
		    vehRestrictions2.maxTourCountPerChain = 1;
		    vehRestrictions2.maxDrivingPeriodPerChain = 28800;
		    vehRestrictions2.maxTransportPointCountPerTour = 10;
		    vehicle2.restrictions = vehRestrictions2;
		    vehicle2.wrappedTransportEquipments = new int[] { 1, 2, 3, 4, 5 };
		    Capacities capacities2 = new Capacities();
		    Quantities quantity2 = new Quantities();
		    quantity2.wrappedQuantities = new int[] { 40, 0, 0, 0, 0, 0, 0 };
		    Quantities[] quantities2 = new Quantities[] { quantity2 };
		    capacities2.wrappedCapacities = quantities2;
		    vehicle2.capacities = capacities2;
		    VehicleCosts costs2 = new VehicleCosts();
		    costs2.costsPerHourDriverOnly = 75.0;
		    costs2.costsPerKm = 18.0;
		    vehicle2.costs = costs2;
		    vehicle2.dimaId = 1000;
            vehicle2.dimaIdSpecified = true;

		    // Step 10.4: Add vehicles to the specified array. Set array to fleet.
		    Vehicle[] vehicles = { vehicle1, vehicle2 };
		    fleet.wrappedVehicles = vehicles;

		    // Step 11.1: Define planning parameters
		    StandardParams planningParams = new StandardParams();

		    // Step 11.2: Add restrictions. See complete list in our API.
		    Restrictions restrictions = new Restrictions();
		    restrictions.maxTransportPointCountPerTour = 10;
		    restrictions.maxTourCount = 2;
		    planningParams.restrictions = restrictions;

		    // Step 11.3: Set properties for distance matrix.
		    DistanceMatrixByRoad distanceMatrixByRoad = new DistanceMatrixByRoad();
		    distanceMatrixByRoad.dimaId = 1000;
		    distanceMatrixByRoad.deleteBeforeUsage = true;
		    distanceMatrixByRoad.deleteAfterUsage = true;
		    DistanceMatrixByRoad[] dimas = new DistanceMatrixByRoad[] { distanceMatrixByRoad };
		    planningParams.wrappedDistanceMatrixCalculation = dimas;

		    // Step 11.4: Set standard properties.
		    planningParams.constructionStep = true;
		    planningParams.improvementStep = true;
		    planningParams.tourReductionStep = false;

		    // Step 12: we don't add an input plan

		    try 
            {
			    // Step 13: call method planBasicTours and add the arguments as
			    // defined above.
                Plan plan = xTourClient.planBasicTours(transportOrders, depots, fleet, planningParams, null, cxt);
                return plan;
		    } catch (Exception e)
            {
                //TODO: proper exception handling
                return null;
            }
	    }
    }

    public class TourTableEntry
    {
        public int vehicleID { get; set; }
        public int tourID { get; set; }
        public int orderID { get; set; }
        public String desc { get; set; }
        public String arrival { get; set; }
        public String departure { get; set; }
        public String service { get; set; }
        public double dist { get; set; }
        public String drivingPeriod { get; set; }
        public double quantitiy { get; set; } 
    } 
}
